unit RunOptions;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TFmRunOptions = class(TForm)
    RgRunType: TRadioGroup;
    CbRepeatDrivers: TCheckBox;
    EdRepeatDriveTime: TEdit;
    LblTimeUnit1: TLabel;
    CbResetStates: TCheckBox;
    EdResetStateTime: TEdit;
    LblTimeUnit2: TLabel;
    BtnClose: TButton;
    MmInfo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure RgRunTypeClick(Sender: TObject);
    procedure CbRepeatDriversClick(Sender: TObject);
    procedure CbResetStatesClick(Sender: TObject);
    procedure EdRepeatDriveTimeExit(Sender: TObject);
    procedure EdResetStateTimeExit(Sender: TObject);
    procedure BtnCloseClick(Sender: TObject);
    procedure EdRepeatDriveTimeChange(Sender: TObject);
    procedure EdResetStateTimeChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FmRunOptions: TFmRunOptions;

implementation

{$R *.DFM}

uses frontend;

procedure TFmRunOptions.FormCreate(Sender: TObject);
begin
 LblTimeUnit1.Caption := ModelDef.timeunit;
 LblTimeUnit2.Caption := ModelDef.timeunit;
end;

procedure TFmRunOptions.RgRunTypeClick(Sender: TObject);
begin
 if RgRunType.ItemIndex = 0 then // Normal run
   begin
     EdRepeatDriveTime.Text := '';
     EdRepeatDriveTime.Enabled := False;
     CbRepeatDrivers.Checked := False;
     CbRepeatDriversClick(RgRunType);
     CbRepeatDrivers.Enabled := False;
     EdResetStateTime.Enabled := False;
     EdResetStateTime.Text := '';
     CbResetStates.Checked := false;
     CbResetStatesClick(RgRunType);
     CbResetStates.Enabled := False;
     MainForm.MINormalRun.Checked := True;
     RunComplete := False;
     Readytorun := False;
   end
 else        // Special run
   begin
     CbRepeatDrivers.Enabled := True;
     CbResetStates.Enabled := True;
     CbRepeatDrivers.State := cbunchecked;
     CbResetStates.Checked := false;
     EdRepeatDriveTime.Enabled := True;
     EdResetStateTime.Enabled := True;
     MainForm.MISpecialRun.Checked := True;
     RunComplete := False;
     Readytorun := False;
   end;
end;

procedure TFmRunOptions.CbRepeatDriversClick(Sender: TObject);
begin
  if CbRepeatDrivers.Checked then
    begin
       MainForm.RunXRepeatedly := True;
    end
  else
    begin
       MainForm.RunXRepeatedly := False;
    end;
end;

procedure TFmRunOptions.CbResetStatesClick(Sender: TObject);
begin
  if CbResetStates.Checked then
    begin
       MainForm.ResetStates := True;
    end
  else
    begin
       MainForm.ResetStates := False;
    end;
end;

procedure TFmRunOptions.EdRepeatDriveTimeExit(Sender: TObject);
begin
try
 if EdRepeatDriveTime.Text <> '' then
   MainForm.RepeatDriveTime := strtoint(EdRepeatDriveTime.Text);
except
 MessageDlg('Invalid driver repeat time. Run set to Normal run.', mtwarning,
      [mbOK], 0);
 MainForm.MINormalRunClick(FmRunOptions);
 FmRunOptions.FormShow(FmRunOptions);
end;
end;

procedure TFmRunOptions.EdResetStateTimeExit(Sender: TObject);
begin
try
 if EdResetStateTime.Text <> '' then
   MainForm.ResetStateTime := strtoint(EdResetStateTime.Text);
except
 MessageDlg('Invalid state variable reset time. Run set to Normal run.',
      mtwarning, [mbOK], 0);
 MainForm.MINormalRunClick(FmRunOptions);
 FmRunOptions.FormShow(FmRunOptions);
end;
end;

procedure TFmRunOptions.BtnCloseClick(Sender: TObject);
begin
  if MainForm.RunXRepeatedly or MainForm.ResetStates then
    MainForm.MISpecialRun.Checked := True
  else
    MainForm.MINormalRun.Checked := True;
  FmRunOptions.Close;
end;

procedure TFmRunOptions.EdRepeatDriveTimeChange(Sender: TObject);
begin
  CbRepeatDrivers.Checked := True;
  CbRepeatDriversClick(EdRepeatDriveTime);
end;

procedure TFmRunOptions.EdResetStateTimeChange(Sender: TObject);
begin
  CbResetStates.Checked := True;
  CbResetStatesClick(EdResetStateTime);
end;

procedure TFmRunOptions.FormShow(Sender: TObject);
begin
 if MainForm.RunXRepeatedly or MainForm.ResetStates then
  begin    // Special Run
   RgRunType.ItemIndex := 1;
   if MainForm.RunXRepeatedly then
     begin
      CbRepeatDrivers.Enabled := True;
      CbRepeatDrivers.Checked := True;
      EdRepeatDriveTime.Enabled := True;
      EdRepeatDriveTime.Text := inttostr(MainForm.RepeatDriveTime);
     end
   else
     begin
      CbRepeatDrivers.Checked := False;
      EdRepeatDriveTime.Text := '';
     end;
   if MainForm.ResetStates then
     begin
      CbResetStates.Enabled := True;
      CbResetStates.Checked := True;
      EdResetStateTime.Enabled := True;
      EdResetStateTime.Text := inttostr(MainForm.ResetStateTime);
     end
   else
     begin
      CbResetStates.Checked := False;
      EdResetStateTime.Text := '';
     end;
  end
 else   // Normal Run
  begin
   RgRunType.ItemIndex := 0;
   CbRepeatDrivers.Enabled := False;
   CbRepeatDrivers.Checked := False;
   EdRepeatDriveTime.Enabled := False;
   EdRepeatDriveTime.Text := '';
   CbResetStates.Enabled := False;
   CbResetStates.Checked := False;
   EdResetStateTime.Enabled := False;
   EdResetStateTime.Text := '';
  end;
{ if MainForm.RunXRepeatedly then
   begin
     CbRepeatDrivers.Enabled := True;
     CbRepeatDrivers.Checked := True;
     EdRepeatDriveTime.Enabled := True;
     EdRepeatDriveTime.Text := inttostr(MainForm.RepeatDriveTime);
   end
 else
   begin
     CbRepeatDrivers.Enabled := False;
     CbRepeatDrivers.Checked := False;
     EdRepeatDriveTime.Enabled := False;
     EdRepeatDriveTime.Text := '';
   end;
 if MainForm.ResetStates then
   begin
     CbResetStates.Enabled := True;
     CbResetStates.Checked := True;
     EdResetStateTime.Enabled := True;
     EdResetStateTime.Text := inttostr(MainForm.ResetStateTime);
   end
 else
   begin
     CbResetStates.Enabled := False;
     CbResetStates.Checked := False;
     EdResetStateTime.Enabled := False;
     EdResetStateTime.Text := '';
   end;                          }
end;

end.

