{ This unit defines the structure of the model. There are four functions. The
  first function, called counts, defines the number, names, and units of the
  model, the state variables, the process variables, the driver variables and
  the parameters. The second function, called processes, is the actual equations
  which make up the model. The third function, derivs, calculates the
  derivatives of state variables. And the fourth function, parcount, is used to
  automatically number the parameters consecutively. 
    The state variables, driver variables, process variables and parameters are
  all stored in global arrays, called stat, drive, proc, and par, respectively.
  The function counts accesses the global arrays directly but the other functions
  operate on copies of the global arrays. }
unit equations;

interface

uses  stypes, math;

PROCEDURE counts;
PROCEDURE processes(time:double; var tdrive:drivearray; var tpar:paramarray;
                       var tstat:statearray;var tproc:processarray;
                       CalculateDiscrete:Boolean);
PROCEDURE derivs(t:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
function ParCount(processnum:integer) : integer;

var
  tproc:processarray;
  tstat:statearray;
  sensflag:boolean;

implementation

uses frontend, calculate;

           { Do not make modifcations above this line. }
{*****************************************************************************}

{ This procedure defines the model. The number of parameters, state, driver and
  process variables are all set in this procedure. The model name, version
  number and time unit are also set here. This procedure accesses the global
  arrays containing the the parameters, state, driver and process variables and
  the global structure ModelDef directly, to save memory space. }
PROCEDURE counts;
var
 i,npar,CurrentProc:integer;
begin
{ Set the modelname, version and time unit. }
ModelDef.modelname := 'Root Model';
ModelDef.versionnumber := '1.0.4';
ModelDef.timeunit := 'second';

{ Set the number of state variables in the model. The maximum number of state
  variables is maxstate, in unit stypes. }
ModelDef.numstate := 30;

{ Enter the name, units and symbol for each state variable. The maximum length
  of the state variable name is 17 characters and the maximum length for units
  and symbol is 20 characters. }
 
 
with stat[1] do
 begin
    name:='Dissolved Org C @0';  units:='ug';  symbol:='DOC1';
 end;
 
with stat[2] do
 begin
    name:='Bacterial carbon @0';  units:='ug';  symbol:='BC1';
 end;
 
with stat[3] do
 begin
    name:='Carbon in predator @0';  units:='ug';  symbol:='PC1';
 end;
 
with stat[4] do
 begin
    name:='Dissolved Org C @4';  units:='ug';  symbol:='DOC2';
 end;
 
with stat[5] do
 begin
    name:='Bacterial carbon @4';  units:='ug';  symbol:='BC2';
 end;
 
with stat[6] do
 begin
    name:='Carbon in predator @4';  units:='ug';  symbol:='PC2';
 end;
 
with stat[7] do
 begin
    name:='Dissolved Org C @8';  units:='ug';  symbol:='DOC3';
 end;
 
with stat[8] do
 begin
    name:='Bacterial carbon @8';  units:='ug';  symbol:='BC3';
 end;
 
with stat[9] do
 begin
    name:='Carbon in predator @8';  units:='ug';  symbol:='PC3';
 end;
 
with stat[10] do
 begin
    name:='Dissolved Org C @12';  units:='ug';  symbol:='DOC4';
 end;
 
with stat[11] do
 begin
    name:='Bacterial carbon @12';  units:='ug';  symbol:='BC4';
 end;
 
with stat[12] do
 begin
    name:='Carbon in predator @12';  units:='ug';  symbol:='PC4';
 end;
 
with stat[13] do
 begin
    name:='Dissolved Org C @16';  units:='ug';  symbol:='DOC5';
 end;
 
with stat[14] do
 begin
    name:='Bacterial carbon @16';  units:='ug';  symbol:='BC5';
 end;
 
with stat[15] do
 begin
    name:='Carbon in predator @16';  units:='ug';  symbol:='PC5';
 end;
 
with stat[16] do
 begin
    name:='Dissolved nitrogen @0';  units:='ug';  symbol:='N1';
 end;
 
with stat[17] do
 begin
    name:='Bacterial Nitrogen @0';  units:='ug';  symbol:='BN1';
 end;
 
with stat[18] do
 begin
    name:='Nitrogen in predator @0';  units:='ug';  symbol:='PN1';
 end;
 
with stat[19] do
 begin
    name:='Dissolved nitrogen @4';  units:='ug';  symbol:='N2';
 end;
 
with stat[20] do
 begin
    name:='Bacterial nitrogen @4';  units:='ug';  symbol:='BN2';
 end;
 
with stat[21] do
 begin
    name:='Nitrogen in predator @4';  units:='ug';  symbol:='PN2';
 end;
 
with stat[22] do
 begin
    name:='Dissolved nitrogen @8';  units:='ug';  symbol:='N3';
 end;
 
with stat[23] do
 begin
    name:='Bacterial nitrogen @8';  units:='ug';  symbol:='BN3';
 end;
 
with stat[24] do
 begin
    name:='Nitrogen in predator @8';  units:='ug';  symbol:='PN3';
 end;
 
with stat[25] do
 begin
    name:='Dissolved nitrogen @12';  units:='ug';  symbol:='N4';
 end;
 
with stat[26] do
 begin
    name:='Bacterial nitrogen @12';  units:='ug';  symbol:='BN4';
 end;
 
with stat[27] do
 begin
    name:='Nitrogen in predator@12';  units:='ug';  symbol:='PN4';
 end;
 
with stat[28] do
 begin
    name:='Dissolved nitrogen @16';  units:='ug';  symbol:='N5';
 end;
 
with stat[29] do
 begin
    name:='Bacterial nitrogen @16';  units:='ug';  symbol:='BN5';
 end;
 
with stat[30] do
 begin
    name:='Nitrogen in predator@16';  units:='ug';  symbol:='PN5';
 end;

{ Set the total number of processes in the model. The first numstate processes
  are the derivatives of the state variables. The maximum number of processes is
  maxparam, in unit stypes. }
ModelDef.numprocess := ModelDef.numstate + 91;

{ For each process, set proc[i].parameters equal to the number of parameters
  associated with that process, and set IsDiscrete to true or false. After each
  process, set the name, units, and symbol for all parameters associated with
  that process. Note that Parcount returns the total number of parameters in
  all previous processes. }
 
CurrentProc := ModelDef.numstate + 1;
With proc[CurrentProc] do
   begin
      name       := 'Diffusion of C in soil';
      units       := 'ug s-1';
      symbol       := 'dC1';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='C diffusion constant';  units:='s-1';  symbol:='a';
 end;
 
CurrentProc := ModelDef.numstate + 2;
With proc[CurrentProc] do
   begin
      name       := 'Diffusion of C to 8';
      units       := 'ug s-1';
      symbol       := 'dC2';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 3;
With proc[CurrentProc] do
   begin
      name       := 'Diffusion of C to 12';
      units       := 'ug s-1';
      symbol       := 'dC3';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 4;
With proc[CurrentProc] do
   begin
      name       := 'Diffusion of C to 16';
      units       := 'ug s-1';
      symbol       := 'dC4';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 5;
With proc[CurrentProc] do
   begin
      name       := 'Diffusion of C out';
      units       := 'ug s-1';
      symbol       := 'dC5';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 6;
With proc[CurrentProc] do
   begin
      name       := 'Carbon uptake @0';
      units       := 'ug s-1';
      symbol       := 'CU1';
      parameters       := 3;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Maximum C uptake';  units:='ug s-1';  symbol:='Cmax';
 end;
with par[npar + 2] do
 begin
    name:='C half saturation';  units:='ug s-1';  symbol:='Kmc';
 end;
with par[npar + 3] do
 begin
    name:='C uptake exponent';  units:='none';  symbol:='beta';
 end;
 
CurrentProc := ModelDef.numstate + 7;
With proc[CurrentProc] do
   begin
      name       := 'Carbon uptake @4';
      units       := 'ug s-1';
      symbol       := 'CU2';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 8;
With proc[CurrentProc] do
   begin
      name       := 'Carbon uptake @8';
      units       := 'ug s-1';
      symbol       := 'CU3';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 9;
With proc[CurrentProc] do
   begin
      name       := 'Carbon uptake @12';
      units       := 'ug s-1';
      symbol       := 'CU4';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 10;
With proc[CurrentProc] do
   begin
      name       := 'Carbon uptake @16';
      units       := 'ug s-1';
      symbol       := 'CU5';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 11;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria C migration to';
      units       := 'ug s-1';
      symbol       := 'mBC1';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Bacterial migration rate';  units:='s-1';  symbol:='m';
 end;
 
CurrentProc := ModelDef.numstate + 12;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria C migr to 8';
      units       := 'ug s-1';
      symbol       := 'mBC2';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 13;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria C migr to12';
      units       := 'ug s-1';
      symbol       := 'mBC3';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 14;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria C migr to16';
      units       := 'ug s-1';
      symbol       := 'mBC4';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 15;
With proc[CurrentProc] do
   begin
      name       := 'Bacterial C migr out';
      units       := 'ug s-1';
      symbol       := 'mBC5';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 16;
With proc[CurrentProc] do
   begin
      name       := 'Bacterial Resp @0';
      units       := 'ug s-1';
      symbol       := 'Rb1';
      parameters       := 2;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Bacteria resp intercept';  units:='s-1';  symbol:='qmin';
 end;
with par[npar + 2] do
 begin
    name:='Bacteria resp slope';  units:='s-1';  symbol:='qo';
 end;
 
CurrentProc := ModelDef.numstate + 17;
With proc[CurrentProc] do
   begin
      name       := 'Bacterial Resp @4';
      units       := 'ug s-1';
      symbol       := 'Rb2';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 18;
With proc[CurrentProc] do
   begin
      name       := 'Bacterial Resp @8';
      units       := 'ug s-1';
      symbol       := 'Rb3';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 19;
With proc[CurrentProc] do
   begin
      name       := 'Bacterial Resp @12';
      units       := 'ug s-1';
      symbol       := 'Rb4';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 20;
With proc[CurrentProc] do
   begin
      name       := 'Bacterial Resp @16';
      units       := 'ug s-1';
      symbol       := 'Rb5';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 21;
With proc[CurrentProc] do
   begin
      name       := 'Carbon assimilation @0';
      units       := 'ug s-1';
      symbol       := 'CC1';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Exploitation efficiency';  units:='none';  symbol:='Ee';
 end;
 
CurrentProc := ModelDef.numstate + 22;
With proc[CurrentProc] do
   begin
      name       := 'C lost by predator @0';
      units       := 'ug s-1';
      symbol       := 'loss1';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Assimilation efficiency';  units:='none';  symbol:='Ea';
 end;
 
CurrentProc := ModelDef.numstate + 23;
With proc[CurrentProc] do
   begin
      name       := 'C lost by predator @4';
      units       := 'ug s-1';
      symbol       := 'loss2';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 24;
With proc[CurrentProc] do
   begin
      name       := 'C lost by predator @8';
      units       := 'ug s-1';
      symbol       := 'loss3';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 25;
With proc[CurrentProc] do
   begin
      name       := 'C lost by predator @12';
      units       := 'ug s-1';
      symbol       := 'loss4';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 26;
With proc[CurrentProc] do
   begin
      name       := 'C lost by predator @16';
      units       := 'ug s-1';
      symbol       := 'loss5';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 27;
With proc[CurrentProc] do
   begin
      name       := 'Carbon assimilation @4';
      units       := 'ug s-1';
      symbol       := 'CC2';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 28;
With proc[CurrentProc] do
   begin
      name       := 'Carbon assimilation @8';
      units       := 'ug s-1';
      symbol       := 'CC3';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 29;
With proc[CurrentProc] do
   begin
      name       := 'Carbon assimilation @12';
      units       := 'ug s-1';
      symbol       := 'CC4';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 30;
With proc[CurrentProc] do
   begin
      name       := 'Carbon assimilation @16';
      units       := 'ug s-1';
      symbol       := 'CC5';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 31;
With proc[CurrentProc] do
   begin
      name       := 'Predator respiration @0';
      units       := 'ug s-1';
      symbol       := 'Rp1';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Pred metabolic resp const';  units:='s-1';  symbol:='rm';
 end;
 
CurrentProc := ModelDef.numstate + 32;
With proc[CurrentProc] do
   begin
      name       := 'Predator respiration @4';
      units       := 'ug s-1';
      symbol       := 'Rp2';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 33;
With proc[CurrentProc] do
   begin
      name       := 'Predator respiration @8';
      units       := 'ug s-1';
      symbol       := 'Rp3';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 34;
With proc[CurrentProc] do
   begin
      name       := 'Predator respiration @1';
      units       := 'ug s-1';
      symbol       := 'Rp4';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 35;
With proc[CurrentProc] do
   begin
      name       := 'Predator respiration @1';
      units       := 'ug s-1';
      symbol       := 'Rp5';
      parameters       := 0;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 36;
With proc[CurrentProc] do
   begin
      name       := 'Predator C migration to';
      units       := 'ug s-1';
      symbol       := 'mP1';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Preditor migration rate';  units:='s-1';  symbol:='mm';
 end;
 
CurrentProc := ModelDef.numstate + 37;
With proc[CurrentProc] do
   begin
      name       := 'Predator C migration to';
      units       := 'ug s-1';
      symbol       := 'mP2';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 38;
With proc[CurrentProc] do
   begin
      name       := 'Predator C migr to 12';
      units       := 'ug s-1';
      symbol       := 'mP3';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 39;
With proc[CurrentProc] do
   begin
      name       := 'Predator C migr to 16';
      units       := 'ug s-1';
      symbol       := 'mP4';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 40;
With proc[CurrentProc] do
   begin
      name       := 'Predator C migration ou';
      units       := 'ug s-1';
      symbol       := 'mP5';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 41;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen diffusion @4';
      units       := 'ug s-1';
      symbol       := 'dN1';
      parameters       := 1;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N diffusion rate';  units:='s-1';  symbol:='b';
 end;
 
CurrentProc := ModelDef.numstate + 42;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen diffusion @8';
      units       := 'ug s-1';
      symbol       := 'dN2';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 43;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen diffusion @12';
      units       := 'ug s-1';
      symbol       := 'dN3';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 44;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen diffusion @16';
      units       := 'ug s-1';
      symbol       := 'dN4';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 45;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen diffusion in';
      units       := 'ug s-1';
      symbol       := 'dN5';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 46;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen uptake to root';
      units       := 'ug s-1';
      symbol       := 'N';
      parameters       := 2;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Max rate of root N uptake';  units:='ug s-1';  symbol:='RNmax';
 end;
with par[npar + 2] do
 begin
    name:='Root N half sat. constant';  units:='ug s-1';  symbol:='RKmn';
 end;
 
CurrentProc := ModelDef.numstate + 47;
With proc[CurrentProc] do
   begin
      name       := 'N uptake by bacteria @0';
      units       := 'ug s-1';
      symbol       := 'NU1';
      parameters       := 3;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Maximum rate of N uptake';  units:='ug s-1';  symbol:='Nmax';
 end;
with par[npar + 2] do
 begin
    name:='N half sat. constant';  units:='ug s-1';  symbol:='Kmn';
 end;
with par[npar + 3] do
 begin
    name:='Bacteria C:N ratio';  units:='none';  symbol:='r';
 end;
 
CurrentProc := ModelDef.numstate + 48;
With proc[CurrentProc] do
   begin
      name       := 'N uptake by bacteria @4';
      units       := 'ug s-1';
      symbol       := 'NU2';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 49;
With proc[CurrentProc] do
   begin
      name       := 'N uptake by bacteria @8';
      units       := 'ug s-1';
      symbol       := 'NU3';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 50;
With proc[CurrentProc] do
   begin
      name       := 'N uptake by bacteria @1';
      units       := 'ug s-1';
      symbol       := 'NU4';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 51;
With proc[CurrentProc] do
   begin
      name       := 'N uptake by bacteria @1';
      units       := 'ug s-1';
      symbol       := 'NU5';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 52;
With proc[CurrentProc] do
   begin
      name       := 'Predator N deposition @';
      units       := 'ug s-1';
      symbol       := 'q1';
      parameters       := 2;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N deposition constant';  units:='ug s-1';  symbol:='dep';
 end;
with par[npar + 2] do
 begin
    name:='Target N:C ratio';  units:='none';  symbol:='PNtarget';
 end;
 
CurrentProc := ModelDef.numstate + 53;
With proc[CurrentProc] do
   begin
      name       := 'Predator N dep @4';
      units       := 'ug s-1';
      symbol       := 'q2';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 54;
With proc[CurrentProc] do
   begin
      name       := 'Predator N dep @8';
      units       := 'ug s-1';
      symbol       := 'q3';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 55;
With proc[CurrentProc] do
   begin
      name       := 'Predator N dep @12';
      units       := 'ug s-1';
      symbol       := 'q4';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 56;
With proc[CurrentProc] do
   begin
      name       := 'Predator N dep @16';
      units       := 'ug s-1';
      symbol       := 'q5';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 57;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen assim @0';
      units       := 'ug s-1';
      symbol       := 'NC1';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 58;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen assim @4';
      units       := 'ug s-1';
      symbol       := 'NC2';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 59;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen assim @8';
      units       := 'ug s-1';
      symbol       := 'NC3';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 60;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen assim @12';
      units       := 'ug s-1';
      symbol       := 'NC4';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 61;
With proc[CurrentProc] do
   begin
      name       := 'Nitrogen assim @16';
      units       := 'ug s-1';
      symbol       := 'NC5';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 62;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N migration 4';
      units       := 'ug s-1';
      symbol       := 'mNB1';
      parameters       := 1;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Bacteria N migration const';  units:='ug s-1';  symbol:='m2';
 end;
 
CurrentProc := ModelDef.numstate + 63;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N migration 8';
      units       := 'ug s-1';
      symbol       := 'mNB2';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 64;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N migration 12';
      units       := 'ug s-1';
      symbol       := 'mNB3';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 65;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N migration 16';
      units       := 'ug s-1';
      symbol       := 'mNB4';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 66;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N migration ou';
      units       := 'ug s-1';
      symbol       := 'mNB5';
      parameters       := 0;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 67;
With proc[CurrentProc] do
   begin
      name       := 'Predator N migration 4';
      units       := 'ug s-1';
      symbol       := 'mNP1';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 68;
With proc[CurrentProc] do
   begin
      name       := 'Predator N migration 8';
      units       := 'ug s-1';
      symbol       := 'mNP2';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 69;
With proc[CurrentProc] do
   begin
      name       := 'Predator N migration 12';
      units       := 'ug s-1';
      symbol       := 'mNP3';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 70;
With proc[CurrentProc] do
   begin
      name       := 'Predator N migration 16';
      units       := 'ug s-1';
      symbol       := 'mNP4';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 71;
With proc[CurrentProc] do
   begin
      name       := 'Predator N migration ou';
      units       := 'ug s-1';
      symbol       := 'mNP5';
      parameters       := 0;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 72;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N loss @ 0';
      units       := 'ug s-1';
      symbol       := 'qB1';
      parameters       := 1;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Bacteria N dep constant';  units:='ug s-1';  symbol:='depB';
 end;
 
CurrentProc := ModelDef.numstate + 73;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N loss @ 4';
      units       := 'ug s-1';
      symbol       := 'qB2';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 74;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N loss @ 8';
      units       := 'ug s-1';
      symbol       := 'qB3';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 75;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N loss @ 12';
      units       := 'ug s-1';
      symbol       := 'qB4';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 76;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria N loss @ 16';
      units       := 'ug s-1';
      symbol       := 'qB5';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 77;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid C @0';
      units       := 'ug s-1';
      symbol       := 'CUs1';
      parameters       := 2;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Uptake solid C constant';  units:='?';  symbol:='alpha';
 end;
with par[npar + 2] do
 begin
    name:='C:N of soil';  units:='none';  symbol:='rsoil';
 end;
 
CurrentProc := ModelDef.numstate + 78;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid C @4';
      units       := 'ug s-1';
      symbol       := 'CUs2';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 79;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid C @8';
      units       := 'ug s-1';
      symbol       := 'CUs3';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 80;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid C @12';
      units       := 'ug s-1';
      symbol       := 'CUs4';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 81;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid C @16';
      units       := 'ug s-1';
      symbol       := 'CUs5';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 82;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid N @0';
      units       := 'ug s-1';
      symbol       := 'NUs1';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 83;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid N @4';
      units       := 'ug s-1';
      symbol       := 'NUs2';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 84;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid N @8';
      units       := 'ug s-1';
      symbol       := 'NUs3';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 85;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid N @12';
      units       := 'ug s-1';
      symbol       := 'NUs4';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 86;
With proc[CurrentProc] do
   begin
      name       := 'Uptake of solid N @16';
      units       := 'ug s-1';
      symbol       := 'NUs5';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 87;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria C:N @0';
      units       := 'none';
      symbol       := 'r1';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 88;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria C:N @4';
      units       := 'none';
      symbol       := 'r2';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 89;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria C:N @8';
      units       := 'none';
      symbol       := 'r3';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 90;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria C:N @12';
      units       := 'none';
      symbol       := 'r4';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;
 
CurrentProc := ModelDef.numstate + 91;
With proc[CurrentProc] do
   begin
      name       := 'Bacteria C:N @16';
      units       := 'none';
      symbol       := 'r5';
      parameters       := 0;
      ptype       := ptGroup4;
      IsDiscrete       := False;
   end;

{ Set the total number of drivers in the model. The maximum number of drivers is
  maxdrive, in unit stypes. }
ModelDef.numdrive := 6;

{ Set the names, units, and symbols of the drivers. The maximum length for the
  name, units and symbol is 20 characters. }
 
with drive[1] do
 begin
    name:='C leaked to soil';  units:='ug s-1';  symbol:='L';
 end;
 
with drive[2] do
 begin
    name:='C concentration outside';  units:='ug';  symbol:='DOCout';
 end;
 
with drive[3] do
 begin
    name:='N concentration in soil';  units:='ug s-1';  symbol:='Nin';
 end;
 
with drive[4] do
 begin
    name:='Bacterial C outside';  units:='ug';  symbol:='BCout';
 end;
 
with drive[5] do
 begin
    name:='Bacterial N outside';  units:='ug';  symbol:='BNout';
 end;
 
with drive[6] do
 begin
    name:='Predator C outside';  units:='ug';  symbol:='PCout';
 end;

{ The first numstate processes are the derivatives of the state variables. The
  code sets the names, units and symbols accordingly.}
for i:= 1 to ModelDef.numstate do proc[i].name:='d'+stat[i].name+'dt';
for i:= 1 to ModelDef.numstate do proc[i].units := stat[i].units + 't-1';
for i:= 1 to ModelDef.numstate do proc[i].symbol := 'd' + stat[i].symbol + 'dt';

{ Code to sum up the total number of parameters in the model. Do not change the
  next few lines. }
ModelDef.numparam := 0;
for i := 1 to ModelDef.NumProcess do
  ModelDef.numparam := ModelDef.numparam + proc[i].parameters;

end; // counts procedure


{ A procedure to calculate the value of all states and processes at the current
  time. This function accesses time, state variables and process variables by
  reference, ie it uses the same array as the calling routine. It does not use
  the global variables time, stat and proc because values calculated during
  integration might later be discarded. It does access the global variables par,
  drive and ModelDef directly because those values are not modified.

  The model equations are written using variable names which correspond to the
  actual name instead of using the global arrays (i.e. SoilWater instead of
  stat[7].value). This makes it necessary to switch all values into local
  variables, do all the calculations and then put everything back into the
  global variables. Lengthy but worth it in terms of readability of the code. }

PROCEDURE processes(time:double; var tdrive:drivearray; var tpar:paramarray;
                       var tstat:statearray;var tproc:processarray;
                       CalculateDiscrete:Boolean);
var
{ List the variable names you are going to use here. Generally, this list
  includes all the symbols you defined in the procedure counts above. The order
  in which you list them does not matter. }
{States}
DOC1, dDOC1dt, 
BC1, dBC1dt, 
PC1, dPC1dt, 
DOC2, dDOC2dt, 
BC2, dBC2dt, 
PC2, dPC2dt, 
DOC3, dDOC3dt, 
BC3, dBC3dt, 
PC3, dPC3dt, 
DOC4, dDOC4dt, 
BC4, dBC4dt, 
PC4, dPC4dt, 
DOC5, dDOC5dt, 
BC5, dBC5dt, 
PC5, dPC5dt, 
N1, dN1dt, 
BN1, dBN1dt, 
PN1, dPN1dt, 
N2, dN2dt, 
BN2, dBN2dt, 
PN2, dPN2dt, 
N3, dN3dt, 
BN3, dBN3dt, 
PN3, dPN3dt, 
N4, dN4dt, 
BN4, dBN4dt, 
PN4, dPN4dt, 
N5, dN5dt, 
BN5, dBN5dt, 
PN5, dPN5dt :double;

{processes and associated parameters}
dC1, a, 
dC2, 
dC3, 
dC4, 
dC5, 
CU1, Cmax, Kmc, beta, 
CU2, 
CU3, 
CU4, 
CU5, 
mBC1, m, 
mBC2, 
mBC3, 
mBC4, 
mBC5, 
Rb1, qmin, qo, 
Rb2, 
Rb3, 
Rb4, 
Rb5, 
CC1, Ee, 
loss1, Ea, 
loss2, 
loss3, 
loss4, 
loss5, 
CC2, 
CC3, 
CC4, 
CC5, 
Rp1, rm, 
Rp2, 
Rp3, 
Rp4, 
Rp5, 
mP1, mm, 
mP2, 
mP3, 
mP4, 
mP5, 
dN1, b, 
dN2, 
dN3, 
dN4, 
dN5, 
N, RNmax, RKmn, 
NU1, Nmax, Kmn, r, 
NU2, 
NU3, 
NU4, 
NU5, 
q1, dep, PNtarget, 
q2, 
q3, 
q4, 
q5, 
NC1, 
NC2, 
NC3, 
NC4, 
NC5, 
mNB1, m2, 
mNB2, 
mNB3, 
mNB4, 
mNB5, 
mNP1, 
mNP2, 
mNP3, 
mNP4, 
mNP5, 
qB1, depB, 
qB2, 
qB3, 
qB4, 
qB5, 
CUs1, alpha, rsoil, 
CUs2, 
CUs3, 
CUs4, 
CUs5, 
NUs1, 
NUs2, 
NUs3, 
NUs4, 
NUs5, 
r1, 
r2, 
r3, 
r4, 
r5, 

{drivers}
L, 
DOCout, 
Nin, 
BCout, 
BNout, 
PCout

{other}

:double;
npar, j, k :integer;

begin
{ Copy the drivers from the global array, drive, into the local variables. }
L := tdrive[1].value;
DOCout := tdrive[2].value;
Nin := tdrive[3].value;
BCout := tdrive[4].value;
BNout := tdrive[5].value;
PCout := tdrive[6].value;

{ Copy the state variables from the global array into the local variables. }
DOC1 := tstat[1].value;
BC1 := tstat[2].value;
PC1 := tstat[3].value;
DOC2 := tstat[4].value;
BC2 := tstat[5].value;
PC2 := tstat[6].value;
DOC3 := tstat[7].value;
BC3 := tstat[8].value;
PC3 := tstat[9].value;
DOC4 := tstat[10].value;
BC4 := tstat[11].value;
PC4 := tstat[12].value;
DOC5 := tstat[13].value;
BC5 := tstat[14].value;
PC5 := tstat[15].value;
N1 := tstat[16].value;
BN1 := tstat[17].value;
PN1 := tstat[18].value;
N2 := tstat[19].value;
BN2 := tstat[20].value;
PN2 := tstat[21].value;
N3 := tstat[22].value;
BN3 := tstat[23].value;
PN3 := tstat[24].value;
N4 := tstat[25].value;
BN4 := tstat[26].value;
PN4 := tstat[27].value;
N5 := tstat[28].value;
BN5 := tstat[29].value;
PN5 := tstat[30].value;

{ And now copy the parameters into the local variables. No need to copy the
  processes from the global array into local variables. Process values will be
  calculated by this procedure.

  Copy the parameters for each process separately using the function ParCount
  to keep track of the number of parameters in the preceeding processes.
  npar now contains the number of parameters in the preceding processes.
  copy the value of the first parameter of this process into it's local
  variable }
npar:=ParCount(ModelDef.numstate + 1);
a := par[npar + 1].value;

npar:=ParCount(ModelDef.numstate + 6);
Cmax := par[npar + 1].value;
Kmc := par[npar + 2].value;
beta := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 11);
m := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 16);
qmin := par[npar + 1].value;
qo := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 21);
Ee := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 22);
Ea := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 31);
rm := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 36);
mm := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 41);
b := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 46);
RNmax := par[npar + 1].value;
RKmn := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 47);
Nmax := par[npar + 1].value;
Kmn := par[npar + 2].value;
r := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 52);
dep := par[npar + 1].value;
PNtarget := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 62);
m2 := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 72);
depB := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 77);
alpha := par[npar + 1].value;
rsoil := par[npar + 2].value;
 
 
{ Enter the equations to calculate the processes here, using the local variable
  names defined above. }

dC1 := a*(DOC1-DOC2);
dC2 := a*(DOC2-DOC3);
dC3 := a*(DOC3-DOC4);
dC4 := a*(DOC4-DOC5);
dC5 := a*(DOC5-DOCout);
CU1 := BN1*Cmax*DOC1/(Kmc+DOC1)*power((r*BN1/BC1),beta);
CU2 := BN2*Cmax*DOC2/(Kmc+DOC2)*power((r*BN2/BC2),beta);
CU3 := BN3*Cmax*DOC3/(Kmc+DOC3)*power((r*BN3/BC3),beta);
CU4 := BN4*Cmax*DOC4/(Kmc+DOC4)*power((r*BN4/BC4),beta);
CU5 := BN5*Cmax*DOC5/(Kmc+DOC5)*power((r*BN5/BC5),beta);
NU1 := BC1*Nmax*N1/(Kmn+N1)*power((1/r*BC1/BN1),beta);
NU2 := BC2*Nmax*N2/(Kmn+N2)*power((1/r*BC2/BN2),beta);
NU3 := BC3*Nmax*N3/(Kmn+N3)*power((1/r*BC3/BN3),beta);
NU4 := BC4*Nmax*N4/(Kmn+N4)*power((1/r*BC4/BN4),beta);
NU5 := BC5*Nmax*N5/(Kmn+N5)*power((1/r*BC5/BN5),beta);
mBC1 := m*(BC1-BC2);
mBC2 := m*(BC2-BC3);
mBC3 := m*(BC3-BC4);
mBC4 := m*(BC4-BC5);
mBC5 := m*(BC5-BCout);
Rb1:= qmin*BC1;
Rb2:= qmin*BC2;
Rb3:= qmin*BC3;
Rb4:= qmin*BC4;
Rb5:= qmin*BC5;
CC1 := Ee*BC1*PC1;
CC2 := Ee*BC2*PC2;
CC3 := Ee*BC3*PC3;
CC4 := Ee*BC4*PC4;
CC5 := Ee*BC5*PC5;
loss1:=(1-Ea)*Ee*BC1;
loss2:=(1-Ea)*Ee*BC2;
loss3:=(1-Ea)*Ee*BC3;
loss4:=(1-Ea)*Ee*BC4;
loss5:=(1-Ea)*Ee*BC5;
Rp1 := rm*PC1+(1-Ea)*CC1;
Rp2 := rm*PC2+(1-Ea)*CC2;
Rp3 := rm*PC3+(1-Ea)*CC3;
Rp4 := rm*PC4+(1-Ea)*CC4;
Rp5 := rm*PC5+(1-Ea)*CC5;
dN1 := b*(N2-N1);
dN2 := b*(N3-N2);
dN3 := b*(N4-N3);
dN4 := b*(N5-N4);
dN5 := b*(NIn-N5);
N := RNmax*N1/(RKmn+N1);
mP1 := mm*(PC1*BC2/BC1-PC2*BC1/BC2);
mP2 := mm*(PC2*BC3/BC2-PC3*BC2/BC3);
mP3 := mm*(PC3*BC4/BC3-PC4*BC3/BC4);
mP4 := mm*(PC4*BC5/BC4-PC5*BC4/BC5);
mP5 := mm*(PC5*BCout/BC5-PCout*BC5/BCout);
if dep*PN1*(PN1/PC1-PNtarget)>0 then q1 := dep*PN1*(PN1/PC1-PNtarget)
    else q1 := 0;
if dep*PN2*(PN2/PC2-PNtarget)>0 then q2 := dep*PN2*(PN2/PC2-PNtarget)
    else q2 := 0;
if dep*PN3*(PN3/PC3-PNtarget)>0 then q3 := dep*PN3*(PN3/PC3-PNtarget)
    else q3 := 0;
if dep*PN4*(PN4/PC4-PNtarget)>0 then q4 := dep*PN4*(PN4/PC4-PNtarget)
    else q4 := 0;
if dep*PN5*(PN5/PC5-PNtarget)>0 then q5 := dep*PN5*(PN5/PC5-PNtarget)
    else q5 := 0;
NC1 := CC1*(BN1/BC1);
NC2 := CC2*(BN2/BC2);
NC3 := CC3*(BN3/BC3);
NC4 := CC4*(BN4/BC4);
NC5 := CC5*(BN5/BC5);
mNB1 := m*(BN1-BN2); 
mNB2 := m*(BN2-BN3); 
mNB3 := m*(BN3-BN4);
mNB4 := m*(BN4-BN5);
mNB5 := m*(BN5-BNout);
mNP1 := mm*(PN1*BC2/BC1-PN2*BC1/BC2);
mNP2 := mm*(PN2*BC3/BC2-PN3*BC2/BC3);
mNP3 := mm*(PN3*BC4/BC3-PN4*BC3/BC4);
mNP4 := mm*(PN4*BC5/BC4-PN5*BC4/BC5);
mNP5 := mm*(PN5*BCout/BC5-PCout*PNtarget*BC5/BCout);
{mNP1 := (PN1/PC1)*mP1;
mNP2 := (PN2/PC2)*mP2;
mNP3 := (PN3/PC3)*mP3;
mNP4 := (PN4/PC4)*mP4;
mNP5 := (PN5/PC5)*mP5;}
qB1 := power(BC1,5)+power(r*BN1,5);
qB2 := power(BC2,5)+power(r*BN2,5);
qB3 := power(BC3,5)+power(r*BN3,5);
qB4 := power(BC4,5)+power(r*BN4,5);
qB5 := power(BC5,5)+power(r*BN5,5);
qB1 := depB*BN1*(1-BC1/power(qB1,0.2));
qB2 := depB*BN2*(1-BC2/power(qB2,0.2));
qB3 := depB*BN3*(1-BC3/power(qB3,0.2));
qB4 := depB*BN4*(1-BC4/power(qB4,0.2));
qB5 := depB*BN5*(1-BC5/power(qB5,0.2));
CUS1:= alpha*sqrt(BC1);
CUS2:= alpha*sqrt(BC2);
CUS3:= alpha*sqrt(BC3);
CUS4:= alpha*sqrt(BC4);
CUS5:= alpha*sqrt(BC5);
NUs1 := CUs1/rsoil;
NUs2 := CUs2/rsoil;
NUs3 := CUs3/rsoil;
NUs4 := CUs4/rsoil;
NUs5 := CUs5/rsoil;
r1 := BC1/BN1;
r2 := BC2/BN2;
r3 := BC3/BN3;
r4 := BC4/BN4;
r5 := BC5/BN5;
if CalculateDiscrete then
begin
// Add any discrete processes here
end; //discrete processes


{ Now calculate the derivatives of the state variables. If the holdConstant
  portion of the state variable is set to true then set the derivative equal to
  zero. }
if tstat[1].HoldConstant = False then
 dDOC1dt := L-dC1-CU1
else
 dDOC1dt := 0;
 
if tstat[2].HoldConstant = False then
 dBC1dt := CU1- mBC1-CC1-Rb1+CUs1
else
 dBC1dt := 0;
 
if tstat[3].HoldConstant = False then
 dPC1dt := CC1-mP1-Rp1
else
 dPC1dt := 0;
 
if tstat[4].HoldConstant = False then
 dDOC2dt := dC1-dC2-CU2
else
 dDOC2dt := 0;
 
if tstat[5].HoldConstant = False then
 dBC2dt := CU2- mBC2-CC2-Rb2+mBC1+CUs2
else
 dBC2dt := 0;
 
if tstat[6].HoldConstant = False then
 dPC2dt := CC2-mP2-Rp2+mP1
else
 dPC2dt := 0;
 
if tstat[7].HoldConstant = False then
 dDOC3dt := dC2-dC3-CU3
else
 dDOC3dt := 0;
 
if tstat[8].HoldConstant = False then
 dBC3dt := CU3- mBC3-CC3-Rb3+mBC2+CUs3
else
 dBC3dt := 0;
 
if tstat[9].HoldConstant = False then
 dPC3dt := CC3-mP3-Rp3+mP2
else
 dPC3dt := 0;
 
if tstat[10].HoldConstant = False then
 dDOC4dt := dC3-dC4-CU4
else
 dDOC4dt := 0;
 
if tstat[11].HoldConstant = False then
 dBC4dt := CU4- mBC4-CC4-Rb4+mBC3+CUs4
else
 dBC4dt := 0;
 
if tstat[12].HoldConstant = False then
 dPC4dt := CC4-mP4-Rp4+mP3
else
 dPC4dt := 0;
 
if tstat[13].HoldConstant = False then
 dDOC5dt := dC4-dC5-CU5
else
 dDOC5dt := 0;
 
if tstat[14].HoldConstant = False then
 dBC5dt := CU5- mBC5-CC5-Rb5+mBC4+CUs5
else
 dBC5dt := 0;
 
if tstat[15].HoldConstant = False then
 dPC5dt := CC5-mP5-Rp5+mP4
else
 dPC5dt := 0;
 
if tstat[16].HoldConstant = False then
 dN1dt := dN1-N-NU1+ q1+qB1
else
 dN1dt := 0;
 
if tstat[17].HoldConstant = False then
 dBN1dt := NU1-mNB1-NC1+NUs1-qB1
else
 dBN1dt := 0;
 
if tstat[18].HoldConstant = False then
 dPN1dt := NC1-mNP1-q1
else
 dPN1dt := 0;
 
if tstat[19].HoldConstant = False then
 dN2dt := dN2-dN1-NU2+q2+qB2
else
 dN2dt := 0;
 
if tstat[20].HoldConstant = False then
 dBN2dt := NU2-mNB2-NC2+mNB1+NUs2-qB2
else
 dBN2dt := 0;
 
if tstat[21].HoldConstant = False then
 dPN2dt := NC2-mNP2-q2+mNP1
else
 dPN2dt := 0;
 
if tstat[22].HoldConstant = False then
 dN3dt := dN3-dN2-NU3+q3+qB3
else
 dN3dt := 0;
 
if tstat[23].HoldConstant = False then
 dBN3dt := NU3-mNB3-NC3+mNB2+NUs3-qB3
else
 dBN3dt := 0;
 
if tstat[24].HoldConstant = False then
 dPN3dt := NC3-mNP3-q3+mNP2
else
 dPN3dt := 0;
 
if tstat[25].HoldConstant = False then
 dN4dt := dN4-dN3-NU4+q4+qB4
else
 dN4dt := 0;
 
if tstat[26].HoldConstant = False then
 dBN4dt := NU4-mNB4-NC4+mNB3+NUs4-qB4
else
 dBN4dt := 0;
 
if tstat[27].HoldConstant = False then
 dPN4dt := NC4-mNP4-q4+mNP3
else
 dPN4dt := 0;
 
if tstat[28].HoldConstant = False then
 dN5dt := dN5-dN4-NU5+q5+qB5
else
 dN5dt := 0;
 
if tstat[29].HoldConstant = False then
 dBN5dt := NU5-mNB5-NC5+mNB4+NUs5-qB5
else
 dBN5dt := 0;
 
if tstat[30].HoldConstant = False then
 dPN5dt := NC5-mNP5-q5+mNP4
else
 dPN5dt := 0;
 

{ Now that the calculations are complete, assign everything back into the arrays
  so the rest of the code can access the values calculated here. (Local variables
  are destroyed at the end of the procedure).

  Put all process values into process variable array. The first numstate
  processes are the derivatives of the state variables (Calculated above).}
tproc[1].value := dDOC1dt;
tproc[2].value := dBC1dt;
tproc[3].value := dPC1dt;
tproc[4].value := dDOC2dt;
tproc[5].value := dBC2dt;
tproc[6].value := dPC2dt;
tproc[7].value := dDOC3dt;
tproc[8].value := dBC3dt;
tproc[9].value := dPC3dt;
tproc[10].value := dDOC4dt;
tproc[11].value := dBC4dt;
tproc[12].value := dPC4dt;
tproc[13].value := dDOC5dt;
tproc[14].value := dBC5dt;
tproc[15].value := dPC5dt;
tproc[16].value := dN1dt;
tproc[17].value := dBN1dt;
tproc[18].value := dPN1dt;
tproc[19].value := dN2dt;
tproc[20].value := dBN2dt;
tproc[21].value := dPN2dt;
tproc[22].value := dN3dt;
tproc[23].value := dBN3dt;
tproc[24].value := dPN3dt;
tproc[25].value := dN4dt;
tproc[26].value := dBN4dt;
tproc[27].value := dPN4dt;
tproc[28].value := dN5dt;
tproc[29].value := dBN5dt;
tproc[30].value := dPN5dt;

{ Now the remaining processes. Be sure to number the processes the same here as
  you did in the procedure counts above. }
tproc[ModelDef.numstate + 1].value := dC1;
tproc[ModelDef.numstate + 2].value := dC2;
tproc[ModelDef.numstate + 3].value := dC3;
tproc[ModelDef.numstate + 4].value := dC4;
tproc[ModelDef.numstate + 5].value := dC5;
tproc[ModelDef.numstate + 6].value := CU1;
tproc[ModelDef.numstate + 7].value := CU2;
tproc[ModelDef.numstate + 8].value := CU3;
tproc[ModelDef.numstate + 9].value := CU4;
tproc[ModelDef.numstate + 10].value := CU5;
tproc[ModelDef.numstate + 11].value := mBC1;
tproc[ModelDef.numstate + 12].value := mBC2;
tproc[ModelDef.numstate + 13].value := mBC3;
tproc[ModelDef.numstate + 14].value := mBC4;
tproc[ModelDef.numstate + 15].value := mBC5;
tproc[ModelDef.numstate + 16].value := Rb1;
tproc[ModelDef.numstate + 17].value := Rb2;
tproc[ModelDef.numstate + 18].value := Rb3;
tproc[ModelDef.numstate + 19].value := Rb4;
tproc[ModelDef.numstate + 20].value := Rb5;
tproc[ModelDef.numstate + 21].value := CC1;
tproc[ModelDef.numstate + 22].value := loss1;
tproc[ModelDef.numstate + 23].value := loss2;
tproc[ModelDef.numstate + 24].value := loss3;
tproc[ModelDef.numstate + 25].value := loss4;
tproc[ModelDef.numstate + 26].value := loss5;
tproc[ModelDef.numstate + 27].value := CC2;
tproc[ModelDef.numstate + 28].value := CC3;
tproc[ModelDef.numstate + 29].value := CC4;
tproc[ModelDef.numstate + 30].value := CC5;
tproc[ModelDef.numstate + 31].value := Rp1;
tproc[ModelDef.numstate + 32].value := Rp2;
tproc[ModelDef.numstate + 33].value := Rp3;
tproc[ModelDef.numstate + 34].value := Rp4;
tproc[ModelDef.numstate + 35].value := Rp5;
tproc[ModelDef.numstate + 36].value := mP1;
tproc[ModelDef.numstate + 37].value := mP2;
tproc[ModelDef.numstate + 38].value := mP3;
tproc[ModelDef.numstate + 39].value := mP4;
tproc[ModelDef.numstate + 40].value := mP5;
tproc[ModelDef.numstate + 41].value := dN1;
tproc[ModelDef.numstate + 42].value := dN2;
tproc[ModelDef.numstate + 43].value := dN3;
tproc[ModelDef.numstate + 44].value := dN4;
tproc[ModelDef.numstate + 45].value := dN5;
tproc[ModelDef.numstate + 46].value := N;
tproc[ModelDef.numstate + 47].value := NU1;
tproc[ModelDef.numstate + 48].value := NU2;
tproc[ModelDef.numstate + 49].value := NU3;
tproc[ModelDef.numstate + 50].value := NU4;
tproc[ModelDef.numstate + 51].value := NU5;
tproc[ModelDef.numstate + 52].value := q1;
tproc[ModelDef.numstate + 53].value := q2;
tproc[ModelDef.numstate + 54].value := q3;
tproc[ModelDef.numstate + 55].value := q4;
tproc[ModelDef.numstate + 56].value := q5;
tproc[ModelDef.numstate + 57].value := NC1;
tproc[ModelDef.numstate + 58].value := NC2;
tproc[ModelDef.numstate + 59].value := NC3;
tproc[ModelDef.numstate + 60].value := NC4;
tproc[ModelDef.numstate + 61].value := NC5;
tproc[ModelDef.numstate + 62].value := mNB1;
tproc[ModelDef.numstate + 63].value := mNB2;
tproc[ModelDef.numstate + 64].value := mNB3;
tproc[ModelDef.numstate + 65].value := mNB4;
tproc[ModelDef.numstate + 66].value := mNB5;
tproc[ModelDef.numstate + 67].value := mNP1;
tproc[ModelDef.numstate + 68].value := mNP2;
tproc[ModelDef.numstate + 69].value := mNP3;
tproc[ModelDef.numstate + 70].value := mNP4;
tproc[ModelDef.numstate + 71].value := mNP5;
tproc[ModelDef.numstate + 72].value := qB1;
tproc[ModelDef.numstate + 73].value := qB2;
tproc[ModelDef.numstate + 74].value := qB3;
tproc[ModelDef.numstate + 75].value := qB4;
tproc[ModelDef.numstate + 76].value := qB5;
tproc[ModelDef.numstate + 77].value := CUs1;
tproc[ModelDef.numstate + 78].value := CUs2;
tproc[ModelDef.numstate + 79].value := CUs3;
tproc[ModelDef.numstate + 80].value := CUs4;
tproc[ModelDef.numstate + 81].value := CUs5;
tproc[ModelDef.numstate + 82].value := NUs1;
tproc[ModelDef.numstate + 83].value := NUs2;
tproc[ModelDef.numstate + 84].value := NUs3;
tproc[ModelDef.numstate + 85].value := NUs4;
tproc[ModelDef.numstate + 86].value := NUs5;
tproc[ModelDef.numstate + 87].value := r1;
tproc[ModelDef.numstate + 88].value := r2;
tproc[ModelDef.numstate + 89].value := r3;
tproc[ModelDef.numstate + 90].value := r4;
tproc[ModelDef.numstate + 91].value := r5;

end;  // End of processes procedure


       { Do not make any modifications to code below this line. }
{****************************************************************************}


{This function counts the parameters in all processes less than processnum.}
function ParCount(processnum:integer) : integer;
var
 NumberofParams, counter : integer;
begin
  NumberofParams := 0;
  for counter := ModelDef.numstate + 1 to processnum - 1 do
         NumberofParams := NumberofParams + proc[counter].parameters;
  ParCount := NumberofParams;
end; // end of parcount function

{ This procedure supplies the derivatives of the state variables to the
  integrator. Since the integrator deals only with the values of the variables
  and not there names, units or the state field HoldConstant, this procedure
  copies the state values into a temporary state array and copies the value of
  HoldConstant into the temporary state array and passes this temporary state
  array to the procedure processes. }
PROCEDURE derivs(t:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
var
   i:integer;
   tempproc:processarray;
   tempstate:statearray;
begin
   tempstate := stat;  // Copy names, units and HoldConstant to tempstate
  // Copy current values of state variables into tempstate
   for i := 1 to ModelDef.numstate do tempstate[i].value := statevalue[i];
  // Calculate the process values
   processes(t,tdrive,tpar,tempstate,tempproc,false);
  // Put process values into dydt array to get passed back to the integrator.
   for i:= 1 to ModelDef.numstate do dydt[i]:=tempproc[i].value;
end;  // end of derivs procedure

end.
