{ This unit defines the structure of the model. There are four functions. The
  first function, called counts, defines the number, names, and units of the
  model, the state variables, the process variables, the driver variables and
  the parameters. The second function, called processes, is the actual equations
  which make up the model. The third function, derivs, calculates the
  derivatives of state variables. And the fourth function, parcount, is used to
  automatically number the parameters consecutively. 
    The state variables, driver variables, process variables and parameters are
  all stored in global arrays, called stat, drive, proc, and par, respectively.
  The function counts accesses the global arrays directly but the other functions
  operate on copies of the global arrays. }
unit equations;

interface

uses  stypes, math;

PROCEDURE counts;
PROCEDURE processes(time:double; var tdrive:drivearray; var tpar:paramarray;
                       var tstat:statearray;var tproc:processarray;
                       CalculateDiscrete:Boolean);
PROCEDURE derivs(t:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
function ParCount(processnum:integer) : integer;

var
  tproc:processarray;
  tstat:statearray;
  sensflag:boolean;

implementation

uses frontend, calculate;

           { Do not make modifcations above this line. }
{*****************************************************************************}

{ This procedure defines the model. The number of parameters, state, driver and
  process variables are all set in this procedure. The model name, version
  number and time unit are also set here. This procedure accesses the global
  arrays containing the the parameters, state, driver and process variables and
  the global structure ModelDef directly, to save memory space. }
PROCEDURE counts;
var
 i,npar,CurrentProc:integer;
begin
{ Set the modelname, version and time unit. }
ModelDef.modelname := 'Final Project C and N';
ModelDef.versionnumber := '1.0.0';
ModelDef.timeunit := 'years';

{ Set the number of state variables in the model. The maximum number of state
  variables is maxstate, in unit stypes. }
ModelDef.numstate := 5;

{ Enter the name, units and symbol for each state variable. The maximum length
  of the state variable name is 17 characters and the maximum length for units
  and symbol is 20 characters. }
 
 
with stat[1] do
 begin
    name:='Inorganic N';  units:='g N m-2';  symbol:='ION';
 end;
 
with stat[2] do
 begin
    name:='Soluble Organic N';  units:='g N m-2';  symbol:='SON';
 end;
 
with stat[3] do
 begin
    name:='N in Vegetation';  units:='g N m-2';  symbol:='VN';
 end;
 
with stat[4] do
 begin
    name:='C in Vegetation';  units:='g C/m-2';  symbol:='VC';
 end;
 
with stat[5] do
 begin
    name:='Soil Organic Carbon';  units:='g C/m-2';  symbol:='SOC';
 end;

{ Set the total number of processes in the model. The first numstate processes
  are the derivatives of the state variables. The maximum number of processes is
  maxparam, in unit stypes. }
ModelDef.numprocess := ModelDef.numstate + 15;

{ For each process, set proc[i].parameters equal to the number of parameters
  associated with that process, and set IsDiscrete to true or false. After each
  process, set the name, units, and symbol for all parameters associated with
  that process. Note that Parcount returns the total number of parameters in
  all previous processes. }
 
CurrentProc := ModelDef.numstate + 1;
With proc[CurrentProc] do
   begin
      name       := 'N Litterfall';
      units       := 'g N m-2 yr-1';
      symbol       := 'LiN';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N Litterfall Parameter';  units:='none';  symbol:='mm';
 end;
 
CurrentProc := ModelDef.numstate + 2;
With proc[CurrentProc] do
   begin
      name       := 'N Leaching SON';
      units       := 'g N m-2 yr-1';
      symbol       := 'LeN';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N Leaching Parameter';  units:='none';  symbol:='oo';
 end;
 
CurrentProc := ModelDef.numstate + 3;
With proc[CurrentProc] do
   begin
      name       := 'N Uptake';
      units       := 'g N m-2 yr-1';
      symbol       := 'UN';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N Uptake Parameter';  units:='none';  symbol:='jj';
 end;
 
CurrentProc := ModelDef.numstate + 4;
With proc[CurrentProc] do
   begin
      name       := 'Net Mineralization';
      units       := 'g N m-2 yr-1';
      symbol       := 'MN';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Net Min Parameter';  units:='none';  symbol:='kk';
 end;
 
CurrentProc := ModelDef.numstate + 5;
With proc[CurrentProc] do
   begin
      name       := 'N Harvest';
      units       := 'g N m-2 yr-1';
      symbol       := 'HN';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N Harvest Parameter';  units:='g C/g N';  symbol:='CNH';
 end;
 
CurrentProc := ModelDef.numstate + 6;
With proc[CurrentProc] do
   begin
      name       := 'N Wind Erosion';
      units       := 'g N m-2 yr-1';
      symbol       := 'EN';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N Wind Erosion';  units:='none';  symbol:='nn';
 end;
 
CurrentProc := ModelDef.numstate + 7;
With proc[CurrentProc] do
   begin
      name       := 'C Litterfall';
      units       := 'g C m-2 yr-1';
      symbol       := 'LiC';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='C Litterfall Parameter';  units:='g C/g N';  symbol:='CNLi';
 end;
 
CurrentProc := ModelDef.numstate + 8;
With proc[CurrentProc] do
   begin
      name       := 'C Leaching';
      units       := 'g C m-2 yr-1';
      symbol       := 'LeC';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='C Leaching Parameter';  units:='none';  symbol:='gg';
 end;
 
CurrentProc := ModelDef.numstate + 9;
With proc[CurrentProc] do
   begin
      name       := 'C Harvest';
      units       := 'C m-2 yr-1';
      symbol       := 'HC';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='C Harvest Parameter';  units:='none';  symbol:='bb';
 end;
 
CurrentProc := ModelDef.numstate + 10;
With proc[CurrentProc] do
   begin
      name       := 'Autotroph Resp';
      units       := 'g C m-2 yr-1';
      symbol       := 'RA';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='A Resp Parameter';  units:='percent';  symbol:='dd';
 end;
 
CurrentProc := ModelDef.numstate + 11;
With proc[CurrentProc] do
   begin
      name       := 'Hetero Resp';
      units       := 'g C m-2 yr-1';
      symbol       := 'RH';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='H Resp Parameter';  units:='none';  symbol:='ee';
 end;
 
CurrentProc := ModelDef.numstate + 12;
With proc[CurrentProc] do
   begin
      name       := 'C Wind Erosion';
      units       := 'g C m-2 yr-1';
      symbol       := 'EC';
      parameters       := 2;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='C Erosion Max';  units:='g C m-2 yr-1';  symbol:='Emax';
 end;
with par[npar + 2] do
 begin
    name:='C Eros Ext Coeff';  units:='mph';  symbol:='Km2';
 end;
 
CurrentProc := ModelDef.numstate + 13;
With proc[CurrentProc] do
   begin
      name       := 'LAI';
      units       := 'm2 m-2';
      symbol       := 'LAI';
      parameters       := 2;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='LAI Exponent';  units:='none';  symbol:='nn';
 end;
with par[npar + 2] do
 begin
    name:='LAI Coeff';  units:='1/g C';  symbol:='beta';
 end;
 
CurrentProc := ModelDef.numstate + 14;
With proc[CurrentProc] do
   begin
      name       := 'Photosynthesis';
      units       := 'g C m-2 yr-1';
      symbol       := 'P';
      parameters       := 3;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='alpha';  units:='none';  symbol:='alpha2';
 end;
with par[npar + 2] do
 begin
    name:='KCO2';  units:='ppm';  symbol:='KCO2';
 end;
with par[npar + 3] do
 begin
    name:='CN Ratio';  units:='grams C/N';  symbol:='CN';
 end;
 
CurrentProc := ModelDef.numstate + 15;
With proc[CurrentProc] do
   begin
      name       := 'N Leaching ION';
      units       := 'g N m-2 yr-1';
      symbol       := 'INloss';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N Leaching ION Parameter';  units:='yr-1';  symbol:='aINloss';
 end;

{ Set the total number of drivers in the model. The maximum number of drivers is
  maxdrive, in unit stypes. }
ModelDef.numdrive := 7;

{ Set the names, units, and symbols of the drivers. The maximum length for the
  name, units and symbol is 20 characters. }
 
with drive[1] do
 begin
    name:='Amount of Fertilizer';  units:='g N yr-1';  symbol:='FN';
 end;
 
with drive[2] do
 begin
    name:='N Deposition';  units:='g N yr-1';  symbol:='DN';
 end;
 
with drive[3] do
 begin
    name:='Harvest';  units:='g N yr-1';  symbol:='HAN';
 end;
 
with drive[4] do
 begin
    name:='CO2 intake';  units:='g C yr-1';  symbol:='CO2';
 end;
 
with drive[5] do
 begin
    name:='Wind Speed';  units:='mph';  symbol:='W';
 end;
 
with drive[6] do
 begin
    name:='Tillage';  units:='none';  symbol:='TT';
 end;
 
with drive[7] do
 begin
    name:='Irradiance';  units:='mjoules/m2/day';  symbol:='Io';
 end;

{ The first numstate processes are the derivatives of the state variables. The
  code sets the names, units and symbols accordingly.}
for i:= 1 to ModelDef.numstate do proc[i].name:='d'+stat[i].name+'dt';
for i:= 1 to ModelDef.numstate do proc[i].units := stat[i].units + 't-1';
for i:= 1 to ModelDef.numstate do proc[i].symbol := 'd' + stat[i].symbol + 'dt';

{ Code to sum up the total number of parameters in the model. Do not change the
  next few lines. }
ModelDef.numparam := 0;
for i := 1 to ModelDef.NumProcess do
  ModelDef.numparam := ModelDef.numparam + proc[i].parameters;

end; // counts procedure


{ A procedure to calculate the value of all states and processes at the current
  time. This function accesses time, state variables and process variables by
  reference, ie it uses the same array as the calling routine. It does not use
  the global variables time, stat and proc because values calculated during
  integration might later be discarded. It does access the global variables par,
  drive and ModelDef directly because those values are not modified.

  The model equations are written using variable names which correspond to the
  actual name instead of using the global arrays (i.e. SoilWater instead of
  stat[7].value). This makes it necessary to switch all values into local
  variables, do all the calculations and then put everything back into the
  global variables. Lengthy but worth it in terms of readability of the code. }

PROCEDURE processes(time:double; var tdrive:drivearray; var tpar:paramarray;
                       var tstat:statearray;var tproc:processarray;
                       CalculateDiscrete:Boolean);
var
{ List the variable names you are going to use here. Generally, this list
  includes all the symbols you defined in the procedure counts above. The order
  in which you list them does not matter. }
{States}
ION, dIONdt, 
SON, dSONdt, 
VN, dVNdt, 
VC, dVCdt, 
SOC, dSOCdt :double;

{processes and associated parameters}
LiN, mm, 
LeN, oo, 
UN, jj, 
MN, kk, 
HN, CNH, 
EN, nn, 
LiC, CNLi, 
LeC, gg, 
HC, bb, 
RA, dd, 
RH, ee, 
EC, Emax, Km2, 
LAI, n, beta,
P, alpha2, KCO2, CN, 
INloss, aINloss, 

{drivers}
FN, 
DN, 
HAN, 
CO2, 
W, 
TT, 
Io

{other}

:double;
npar, j, k :integer;

begin
{ Copy the drivers from the global array, drive, into the local variables. }
FN := tdrive[1].value;
DN := tdrive[2].value;
HAN := tdrive[3].value;
CO2 := tdrive[4].value;
W := tdrive[5].value;
TT := tdrive[6].value;
Io := tdrive[7].value;

{ Copy the state variables from the global array into the local variables. }
ION := tstat[1].value;
SON := tstat[2].value;
VN := tstat[3].value;
VC := tstat[4].value;
SOC := tstat[5].value;

{ And now copy the parameters into the local variables. No need to copy the
  processes from the global array into local variables. Process values will be
  calculated by this procedure.

  Copy the parameters for each process separately using the function ParCount
  to keep track of the number of parameters in the preceeding processes.
  npar now contains the number of parameters in the preceding processes.
  copy the value of the first parameter of this process into it's local
  variable }
npar:=ParCount(ModelDef.numstate + 1);
mm := par[npar + 1].value;

npar:=ParCount(ModelDef.numstate + 2);
oo := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 3);
jj := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 4);
kk := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 5);
CNH := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 6);
nn := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 7);
CNLi := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 8);
gg := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 9);
bb := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 10);
dd := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 11);
ee := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 12);
Emax := par[npar + 1].value;
Km2 := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 13);
nn := par[npar + 1].value;
beta := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 14);
alpha2 := par[npar + 1].value;
KCO2 := par[npar + 2].value;
CN := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 15);
aINloss := par[npar + 1].value;
 
 
{ Enter the equations to calculate the processes here, using the local variable
  names defined above. }

LeN := oo*SON;
INloss := aINloss*ION;
UN := jj*ION;
MN := (kk*SON)*exp(-2.56*ION);
EN := ((Emax*W)/(Km2+W))*SON;
LiC := CNLi *VC;
LiN := (1-mm)*LiC*VN/VC;
LeC := LeN*SOC/SON;
HC := bb*VC*HAN;
HN := HC*VN/VC;
RA := (VC*dd);
RH := (ee+TT-ee*TT)*SOC;
EC := ((Emax*W)/(Km2 + W))*SOC;
if VC>0 then LAI := power(beta*VC, n) else LAI:=0;
P := alpha2*Io*(1-exp(-LAI/2))*CO2/(KCO2 + CO2);
IF UN*CN > P - RA then UN := (P-RA)/CN  else P := UN*CN + RA;
if CalculateDiscrete then
begin
// Add any discrete processes here
end; //discrete processes


{ Now calculate the derivatives of the state variables. If the holdConstant
  portion of the state variable is set to true then set the derivative equal to
  zero. }
if tstat[1].HoldConstant = False then
 DIONdt := FN + DN + MN -UN - INloss
else
 dIONdt := 0;
 
if tstat[2].HoldConstant = False then
 DSONdt := LiN -EN -LeN -MN
else
 dSONdt := 0;
 
if tstat[3].HoldConstant = False then
 DVNdt := UN-LiN-HN
else
 dVNdt := 0;
 
if tstat[4].HoldConstant = False then
 dVCdt:= P-RA-LiC-HC
else
 dVCdt := 0;
 
if tstat[5].HoldConstant = False then
 dSOCdt := liC-RH-leC-EC
else
 dSOCdt := 0;
 

{ Now that the calculations are complete, assign everything back into the arrays
  so the rest of the code can access the values calculated here. (Local variables
  are destroyed at the end of the procedure).

  Put all process values into process variable array. The first numstate
  processes are the derivatives of the state variables (Calculated above).}
tproc[1].value := dIONdt;
tproc[2].value := dSONdt;
tproc[3].value := dVNdt;
tproc[4].value := dVCdt;
tproc[5].value := dSOCdt;

{ Now the remaining processes. Be sure to number the processes the same here as
  you did in the procedure counts above. }
tproc[ModelDef.numstate + 1].value := LiN;
tproc[ModelDef.numstate + 2].value := LeN;
tproc[ModelDef.numstate + 3].value := UN;
tproc[ModelDef.numstate + 4].value := MN;
tproc[ModelDef.numstate + 5].value := HN;
tproc[ModelDef.numstate + 6].value := EN;
tproc[ModelDef.numstate + 7].value := LiC;
tproc[ModelDef.numstate + 8].value := LeC;
tproc[ModelDef.numstate + 9].value := HC;
tproc[ModelDef.numstate + 10].value := RA;
tproc[ModelDef.numstate + 11].value := RH;
tproc[ModelDef.numstate + 12].value := EC;
tproc[ModelDef.numstate + 13].value := LAI;
tproc[ModelDef.numstate + 14].value := P;
tproc[ModelDef.numstate + 15].value := INloss;

end;  // End of processes procedure


       { Do not make any modifications to code below this line. }
{****************************************************************************}


{This function counts the parameters in all processes less than processnum.}
function ParCount(processnum:integer) : integer;
var
 NumberofParams, counter : integer;
begin
  NumberofParams := 0;
  for counter := ModelDef.numstate + 1 to processnum - 1 do
         NumberofParams := NumberofParams + proc[counter].parameters;
  ParCount := NumberofParams;
end; // end of parcount function

{ This procedure supplies the derivatives of the state variables to the
  integrator. Since the integrator deals only with the values of the variables
  and not there names, units or the state field HoldConstant, this procedure
  copies the state values into a temporary state array and copies the value of
  HoldConstant into the temporary state array and passes this temporary state
  array to the procedure processes. }
PROCEDURE derivs(t:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
var
   i:integer;
   tempproc:processarray;
   tempstate:statearray;
begin
   tempstate := stat;  // Copy names, units and HoldConstant to tempstate
  // Copy current values of state variables into tempstate
   for i := 1 to ModelDef.numstate do tempstate[i].value := statevalue[i];
  // Calculate the process values
   processes(t,tdrive,tpar,tempstate,tempproc,false);
  // Put process values into dydt array to get passed back to the integrator.
   for i:= 1 to ModelDef.numstate do dydt[i]:=tempproc[i].value;
end;  // end of derivs procedure

end.
