{ This unit defines the structure of the model. There are four functions. The
  first function, called counts, defines the number, names, and units of the
  model, the state variables, the process variables, the driver variables and
  the parameters. The second function, called processes, is the actual equations
  which make up the model. The third function, derivs, calculates the
  derivatives of state variables. And the fourth function, parcount, is used to
  automatically number the parameters consecutively. 
    The state variables, driver variables, process variables and parameters are
  all stored in global arrays, called stat, drive, proc, and par, respectively.
  The function counts accesses the global arrays directly but the other functions
  operate on copies of the global arrays. }
unit equations;

interface

uses  stypes, math;

PROCEDURE counts;
PROCEDURE processes(time:double; var tdrive:drivearray; var tpar:paramarray;
                       var tstat:statearray;var tproc:processarray;
                       CalculateDiscrete:Boolean);
PROCEDURE derivs(t:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
function ParCount(processnum:integer) : integer;

var
  tproc:processarray;
  tstat:statearray;
  sensflag:boolean;

implementation

uses frontend, calculate;

           { Do not make modifcations above this line. }
{*****************************************************************************}

{ This procedure defines the model. The number of parameters, state, driver and
  process variables are all set in this procedure. The model name, version
  number and time unit are also set here. This procedure accesses the global
  arrays containing the the parameters, state, driver and process variables and
  the global structure ModelDef directly, to save memory space. }
PROCEDURE counts;
var
 i,npar,CurrentProc:integer;
begin
{ Set the modelname, version and time unit. }
ModelDef.modelname := 'PCB model';
ModelDef.versionnumber := '1.1.1';
ModelDef.timeunit := 'years';

{ Set the number of state variables in the model. The maximum number of state
  variables is maxstate, in unit stypes. }
ModelDef.numstate := 13;

{ Enter the name, units and symbol for each state variable. The maximum length
  of the state variable name is 17 characters and the maximum length for units
  and symbol is 20 characters. }
 
 
with stat[1] do
 begin
    name:='PCB 2nd Carnivores';  units:='g PCB g C-1';  symbol:='SCpcb';
 end;
 
with stat[2] do
 begin
    name:='PCB 1st Carnivores';  units:='g PCB g C-1';  symbol:='FCpcb';
 end;
 
with stat[3] do
 begin
    name:='PCB herbivores';  units:='g PCB g C-1';  symbol:='Hpcb';
 end;
 
with stat[4] do
 begin
    name:='PCB detritivores';  units:='g PCB g C-1';  symbol:='Dpcb';
 end;
 
with stat[5] do
 begin
    name:='PCB Detritus';  units:='g PCB g C-1';  symbol:='Lpcb';
 end;
 
with stat[6] do
 begin
    name:='PCB Primary Prod';  units:='g PCB g C-1';  symbol:='PPpcb';
 end;
 
with stat[7] do
 begin
    name:='PCB Water';  units:='g PCB g C-1';  symbol:='W';
 end;
 
with stat[8] do
 begin
    name:='C 2nd Carnivores';  units:='g C m-2';  symbol:='SCc';
 end;
 
with stat[9] do
 begin
    name:='C 1st Carnivores';  units:='g C m-2';  symbol:='FCc';
 end;
 
with stat[10] do
 begin
    name:='C herbivores';  units:='g C m-2';  symbol:='Hc';
 end;
 
with stat[11] do
 begin
    name:='C detritivores';  units:='g C m-2';  symbol:='Dc';
 end;
 
with stat[12] do
 begin
    name:='C Detritus';  units:='g C m-2';  symbol:='Lc';
 end;
 
with stat[13] do
 begin
    name:='C Primary Prod';  units:='g C m-2';  symbol:='PPc';
 end;

{ Set the total number of processes in the model. The first numstate processes
  are the derivatives of the state variables. The maximum number of processes is
  maxparam, in unit stypes. }
ModelDef.numprocess := ModelDef.numstate + 29;

{ For each process, set proc[i].parameters equal to the number of parameters
  associated with that process, and set IsDiscrete to true or false. After each
  process, set the name, units, and symbol for all parameters associated with
  that process. Note that Parcount returns the total number of parameters in
  all previous processes. }
 
CurrentProc := ModelDef.numstate + 1;
With proc[CurrentProc] do
   begin
      name       := 'UptakePCBsbyPhoto';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'Ppcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='a';  units:='yr-1';  symbol:='a';
 end;
 
CurrentProc := ModelDef.numstate + 2;
With proc[CurrentProc] do
   begin
      name       := 'GrazebyHerbPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'Gpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='b';  units:='m2 g C-1';  symbol:='b';
 end;
 
CurrentProc := ModelDef.numstate + 3;
With proc[CurrentProc] do
   begin
      name       := '1CarnEatHerbPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'HEpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='c';  units:='yr-1';  symbol:='c';
 end;
 
CurrentProc := ModelDef.numstate + 4;
With proc[CurrentProc] do
   begin
      name       := '1CarnEatDetrPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'DEpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='d';  units:='yr-1';  symbol:='d';
 end;
 
CurrentProc := ModelDef.numstate + 5;
With proc[CurrentProc] do
   begin
      name       := '2CarnEat1CarnPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'FCEpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='e';  units:='yr-1';  symbol:='e';
 end;
 
CurrentProc := ModelDef.numstate + 6;
With proc[CurrentProc] do
   begin
      name       := 'DeathPrimProdPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'PPDpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='f';  units:='yr-1';  symbol:='f';
 end;
 
CurrentProc := ModelDef.numstate + 7;
With proc[CurrentProc] do
   begin
      name       := 'DeathHerbPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'HDpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='g';  units:='yr-1';  symbol:='g';
 end;
 
CurrentProc := ModelDef.numstate + 8;
With proc[CurrentProc] do
   begin
      name       := 'Death1CarnPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'FCDpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='h';  units:='yr-1';  symbol:='h';
 end;
 
CurrentProc := ModelDef.numstate + 9;
With proc[CurrentProc] do
   begin
      name       := 'Death2CarnPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'SCDpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='y';  units:='yr-1';  symbol:='y';
 end;
 
CurrentProc := ModelDef.numstate + 10;
With proc[CurrentProc] do
   begin
      name       := 'DeathDetriPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'DDpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='zz';  units:='yr-1';  symbol:='zz';
 end;
 
CurrentProc := ModelDef.numstate + 11;
With proc[CurrentProc] do
   begin
      name       := 'DetritEatDetriPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'DEDpcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='yz';  units:='yr-1';  symbol:='yz';
 end;
 
CurrentProc := ModelDef.numstate + 12;
With proc[CurrentProc] do
   begin
      name       := 'SedimentPCB';
      units       := 'g PCB g C-1 yr-1';
      symbol       := 'Spcb';
      parameters       := 1;
      ptype       := ptGroup1;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='l';  units:='yr-1';  symbol:='l';
 end;
 
CurrentProc := ModelDef.numstate + 13;
With proc[CurrentProc] do
   begin
      name       := 'GrazeHerbC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'Gc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='m';  units:='yr-1';  symbol:='m';
 end;
 
CurrentProc := ModelDef.numstate + 14;
With proc[CurrentProc] do
   begin
      name       := 'Carn1EatHerbC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'HEc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='n';  units:='yr-1';  symbol:='n';
 end;
 
CurrentProc := ModelDef.numstate + 15;
With proc[CurrentProc] do
   begin
      name       := 'Carn1EatDetC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'DEc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='o';  units:='yr-1';  symbol:='o';
 end;
 
CurrentProc := ModelDef.numstate + 16;
With proc[CurrentProc] do
   begin
      name       := '2CarnEat1CarnC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'FCEc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='p';  units:='yr-1';  symbol:='p';
 end;
 
CurrentProc := ModelDef.numstate + 17;
With proc[CurrentProc] do
   begin
      name       := 'DeathPrimProdC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'PPDc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='q';  units:='yr-1';  symbol:='q';
 end;
 
CurrentProc := ModelDef.numstate + 18;
With proc[CurrentProc] do
   begin
      name       := 'DeathHerbC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'HDc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='r';  units:='yr-1';  symbol:='r';
 end;
 
CurrentProc := ModelDef.numstate + 19;
With proc[CurrentProc] do
   begin
      name       := 'Death1CarnC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'FCDc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='s';  units:='yr-1';  symbol:='s';
 end;
 
CurrentProc := ModelDef.numstate + 20;
With proc[CurrentProc] do
   begin
      name       := 'Death2CarnC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'SCDc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='t';  units:='yr-1';  symbol:='t';
 end;
 
CurrentProc := ModelDef.numstate + 21;
With proc[CurrentProc] do
   begin
      name       := 'DeathDetriC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'DDc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='u';  units:='yr-1';  symbol:='u';
 end;
 
CurrentProc := ModelDef.numstate + 22;
With proc[CurrentProc] do
   begin
      name       := 'DetriEatDetriC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'DEDc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='v';  units:='yr-1';  symbol:='v';
 end;
 
CurrentProc := ModelDef.numstate + 23;
With proc[CurrentProc] do
   begin
      name       := 'SedimentationC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'Sc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='x';  units:='yr-1';  symbol:='x';
 end;
 
CurrentProc := ModelDef.numstate + 24;
With proc[CurrentProc] do
   begin
      name       := 'RespBy2CarnC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'RSCc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='aa';  units:='yr-1';  symbol:='aa';
 end;
 
CurrentProc := ModelDef.numstate + 25;
With proc[CurrentProc] do
   begin
      name       := 'RespBy1CarnC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'RFCc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='bb';  units:='yr-1';  symbol:='bb';
 end;
 
CurrentProc := ModelDef.numstate + 26;
With proc[CurrentProc] do
   begin
      name       := 'RespByHerbC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'RHc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='cc';  units:='yr-1';  symbol:='cc';
 end;
 
CurrentProc := ModelDef.numstate + 27;
With proc[CurrentProc] do
   begin
      name       := 'RespByPrimProdC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'RPPc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='dd';  units:='yr-1';  symbol:='dd';
 end;
 
CurrentProc := ModelDef.numstate + 28;
With proc[CurrentProc] do
   begin
      name       := 'RespByDetriC';
      units       := 'g C g m-2 yr-1';
      symbol       := 'RDc';
      parameters       := 1;
      ptype       := ptGroup2;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='ee';  units:='yr-1';  symbol:='ee';
 end;
 
CurrentProc := ModelDef.numstate + 29;
With proc[CurrentProc] do
   begin
      name       := 'PhotosynthesisC';
      units       := 'g C m-2 yr-1';
      symbol       := 'Pc';
      parameters       := 3;
      ptype       := ptGroup3;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='RatioIrrAbsorbToGPP';  units:='gC Mjoules-1';  symbol:='alphaPc';
 end;
with par[npar + 2] do
 begin
    name:='LightAttenFromWater';  units:='m-1';  symbol:='kw';
 end;
with par[npar + 3] do
 begin
    name:='LightAttenFromPhyto';  units:='m-1';  symbol:='kp';
 end;

{ Set the total number of drivers in the model. The maximum number of drivers is
  maxdrive, in unit stypes. }
ModelDef.numdrive := 3;

{ Set the names, units, and symbols of the drivers. The maximum length for the
  name, units and symbol is 20 characters. }
 
with drive[1] do
 begin
    name:='Human Input PCB';  units:='g PCB g C-1 yr-1';  symbol:='HIpcb';
 end;
 
with drive[2] do
 begin
    name:='IrrEnteringH2Ocol';  units:='Mjoules m-2 yr-1';  symbol:='I0';
 end;
 
with drive[3] do
 begin
    name:='DepthOfWaterCol';  units:='m';  symbol:='z';
 end;

{ The first numstate processes are the derivatives of the state variables. The
  code sets the names, units and symbols accordingly.}
for i:= 1 to ModelDef.numstate do proc[i].name:='d'+stat[i].name+'dt';
for i:= 1 to ModelDef.numstate do proc[i].units := stat[i].units + 't-1';
for i:= 1 to ModelDef.numstate do proc[i].symbol := 'd' + stat[i].symbol + 'dt';

{ Code to sum up the total number of parameters in the model. Do not change the
  next few lines. }
ModelDef.numparam := 0;
for i := 1 to ModelDef.NumProcess do
  ModelDef.numparam := ModelDef.numparam + proc[i].parameters;

end; // counts procedure


{ A procedure to calculate the value of all states and processes at the current
  time. This function accesses time, state variables and process variables by
  reference, ie it uses the same array as the calling routine. It does not use
  the global variables time, stat and proc because values calculated during
  integration might later be discarded. It does access the global variables par,
  drive and ModelDef directly because those values are not modified.

  The model equations are written using variable names which correspond to the
  actual name instead of using the global arrays (i.e. SoilWater instead of
  stat[7].value). This makes it necessary to switch all values into local
  variables, do all the calculations and then put everything back into the
  global variables. Lengthy but worth it in terms of readability of the code. }

PROCEDURE processes(time:double; var tdrive:drivearray; var tpar:paramarray;
                       var tstat:statearray;var tproc:processarray;
                       CalculateDiscrete:Boolean);
var
{ List the variable names you are going to use here. Generally, this list
  includes all the symbols you defined in the procedure counts above. The order
  in which you list them does not matter. }
{States}
SCpcb, dSCpcbdt, 
FCpcb, dFCpcbdt, 
Hpcb, dHpcbdt, 
Dpcb, dDpcbdt, 
Lpcb, dLpcbdt, 
PPpcb, dPPpcbdt, 
W, dWdt, 
SCc, dSCcdt, 
FCc, dFCcdt, 
Hc, dHcdt, 
Dc, dDcdt, 
Lc, dLcdt, 
PPc, dPPcdt :double;

{processes and associated parameters}
Ppcb, a, 
Gpcb, b, 
HEpcb, c, 
DEpcb, d, 
FCEpcb, e, 
PPDpcb, f, 
HDpcb, g, 
FCDpcb, h, 
SCDpcb, y, 
DDpcb, zz, 
DEDpcb, yz, 
Spcb, l, 
Gc, m, 
HEc, n, 
DEc, o, 
FCEc, p, 
PPDc, q, 
HDc, r, 
FCDc, s, 
SCDc, t, 
DDc, u, 
DEDc, v, 
Sc, x, 
RSCc, aa, 
RFCc, bb, 
RHc, cc, 
RPPc, dd, 
RDc, ee, 
Pc, alphaPc, kw, kp, 

{drivers}
HIpcb, 
I0, 
z

{other}

:double;
npar, j, k :integer;

begin
{ Copy the drivers from the global array, drive, into the local variables. }
HIpcb := tdrive[1].value;
I0 := tdrive[2].value;
z := tdrive[3].value;

{ Copy the state variables from the global array into the local variables. }
SCpcb := tstat[1].value;
FCpcb := tstat[2].value;
Hpcb := tstat[3].value;
Dpcb := tstat[4].value;
Lpcb := tstat[5].value;
PPpcb := tstat[6].value;
W := tstat[7].value;
SCc := tstat[8].value;
FCc := tstat[9].value;
Hc := tstat[10].value;
Dc := tstat[11].value;
Lc := tstat[12].value;
PPc := tstat[13].value;

{ And now copy the parameters into the local variables. No need to copy the
  processes from the global array into local variables. Process values will be
  calculated by this procedure.

  Copy the parameters for each process separately using the function ParCount
  to keep track of the number of parameters in the preceeding processes.
  npar now contains the number of parameters in the preceding processes.
  copy the value of the first parameter of this process into it's local
  variable }
npar:=ParCount(ModelDef.numstate + 1);
a := par[npar + 1].value;

npar:=ParCount(ModelDef.numstate + 2);
b := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 3);
c := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 4);
d := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 5);
e := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 6);
f := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 7);
g := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 8);
h := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 9);
y := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 10);
zz := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 11);
yz := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 12);
l := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 13);
m := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 14);
n := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 15);
o := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 16);
p := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 17);
q := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 18);
r := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 19);
s := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 20);
t := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 21);
u := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 22);
v := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 23);
x := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 24);
aa := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 25);
bb := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 26);
cc := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 27);
dd := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 28);
ee := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 29);
alphaPc := par[npar + 1].value;
kw := par[npar + 2].value;
kp := par[npar + 3].value;
 
 
{ Enter the equations to calculate the processes here, using the local variable
  names defined above. }

Gc:=m*PPc;
Ppcb:=a*W;
Gpcb:=Gc*PPpcb*b; 
HEpcb:=c*Hpcb;
DEpcb:=d*Dpcb;
FCEpcb:=e*FCpcb;
PPDpcb:=f*PPpcb;
HDpcb:=g*Hpcb;
FCDpcb:=h*FCpcb;
SCDpcb:=y*SCpcb;
DDpcb:=zz*Dpcb;
DEDpcb:=yz*Lpcb;
Spcb:=l*Lpcb;
HEc:=n*Hc; 
DEc:=o*Dc;
FCEc:=p*FCc;
PPDc:=q*PPc;
HDc:=r*Hc;
FCDc:=s*FCc;
SCDc:=t*SCc;
DDc:=u*Dc;
DEDc:=v*Lc;
Sc:=x*Lc;
RSCc:=aa*SCc;
RFCc:=bb*FCc;
RHc:=cc*Hc;
RPPc:=dd*PPc;
RDc:=ee*Dc;
Pc:=alphaPc*I0*(exp(-kw*z))*(1-(exp(-kp*z)));
if CalculateDiscrete then
begin
// Add any discrete processes here
end; //discrete processes


{ Now calculate the derivatives of the state variables. If the holdConstant
  portion of the state variable is set to true then set the derivative equal to
  zero. }
if tstat[1].HoldConstant = False then
 dSCpcbdt:=FCEpcb-SCDpcb
else
 dSCpcbdt := 0;
 
if tstat[2].HoldConstant = False then
 dFCpcbdt:=HEpcb-FCDpcb+DEpcb-FCEpcb
else
 dFCpcbdt := 0;
 
if tstat[3].HoldConstant = False then
 dHpcbdt:=Gpcb-HDpcb-HEpcb
else
 dHpcbdt := 0;
 
if tstat[4].HoldConstant = False then
 dDpcbdt:=DEDpcb-DDpcb-DEpcb
else
 dDpcbdt := 0;
 
if tstat[5].HoldConstant = False then
 dLpcbdt:=SCDpcb+PPDpcb+HDpcb+FCDpcb-DEDpcb+DDpcb-Spcb
else
 dLpcbdt := 0;
 
if tstat[6].HoldConstant = False then
 dPPpcbdt:=Ppcb-Gpcb-PPDpcb
else
 dPPpcbdt := 0;
 
if tstat[7].HoldConstant = False then
 dWdt:=HIpcb-Ppcb
else
 dWdt := 0;
 
if tstat[8].HoldConstant = False then
 dSCcdt:=FCEc-SCDc-RSCc
else
 dSCcdt := 0;
 
if tstat[9].HoldConstant = False then
 dFCcdt:=HEc-FCDc+DEc-FCEc-RFCc
else
 dFCcdt := 0;
 
if tstat[10].HoldConstant = False then
 dHcdt:=Gc-HDc-HEc-RHc
else
 dHcdt := 0;
 
if tstat[11].HoldConstant = False then
 dDcdt:=DEDc-DDc-DEc-RDc
else
 dDcdt := 0;
 
if tstat[12].HoldConstant = False then
 dLcdt:=SCDc-Sc+PPDc+HDc+FCDc-DEDc+DDc
else
 dLcdt := 0;
 
if tstat[13].HoldConstant = False then
 dPPcdt:=Pc-Gc-PPDc-RPPc
else
 dPPcdt := 0;
 

{ Now that the calculations are complete, assign everything back into the arrays
  so the rest of the code can access the values calculated here. (Local variables
  are destroyed at the end of the procedure).

  Put all process values into process variable array. The first numstate
  processes are the derivatives of the state variables (Calculated above).}
tproc[1].value := dSCpcbdt;
tproc[2].value := dFCpcbdt;
tproc[3].value := dHpcbdt;
tproc[4].value := dDpcbdt;
tproc[5].value := dLpcbdt;
tproc[6].value := dPPpcbdt;
tproc[7].value := dWdt;
tproc[8].value := dSCcdt;
tproc[9].value := dFCcdt;
tproc[10].value := dHcdt;
tproc[11].value := dDcdt;
tproc[12].value := dLcdt;
tproc[13].value := dPPcdt;

{ Now the remaining processes. Be sure to number the processes the same here as
  you did in the procedure counts above. }
tproc[ModelDef.numstate + 1].value := Ppcb;
tproc[ModelDef.numstate + 2].value := Gpcb;
tproc[ModelDef.numstate + 3].value := HEpcb;
tproc[ModelDef.numstate + 4].value := DEpcb;
tproc[ModelDef.numstate + 5].value := FCEpcb;
tproc[ModelDef.numstate + 6].value := PPDpcb;
tproc[ModelDef.numstate + 7].value := HDpcb;
tproc[ModelDef.numstate + 8].value := FCDpcb;
tproc[ModelDef.numstate + 9].value := SCDpcb;
tproc[ModelDef.numstate + 10].value := DDpcb;
tproc[ModelDef.numstate + 11].value := DEDpcb;
tproc[ModelDef.numstate + 12].value := Spcb;
tproc[ModelDef.numstate + 13].value := Gc;
tproc[ModelDef.numstate + 14].value := HEc;
tproc[ModelDef.numstate + 15].value := DEc;
tproc[ModelDef.numstate + 16].value := FCEc;
tproc[ModelDef.numstate + 17].value := PPDc;
tproc[ModelDef.numstate + 18].value := HDc;
tproc[ModelDef.numstate + 19].value := FCDc;
tproc[ModelDef.numstate + 20].value := SCDc;
tproc[ModelDef.numstate + 21].value := DDc;
tproc[ModelDef.numstate + 22].value := DEDc;
tproc[ModelDef.numstate + 23].value := Sc;
tproc[ModelDef.numstate + 24].value := RSCc;
tproc[ModelDef.numstate + 25].value := RFCc;
tproc[ModelDef.numstate + 26].value := RHc;
tproc[ModelDef.numstate + 27].value := RPPc;
tproc[ModelDef.numstate + 28].value := RDc;
tproc[ModelDef.numstate + 29].value := Pc;

end;  // End of processes procedure


       { Do not make any modifications to code below this line. }
{****************************************************************************}


{This function counts the parameters in all processes less than processnum.}
function ParCount(processnum:integer) : integer;
var
 NumberofParams, counter : integer;
begin
  NumberofParams := 0;
  for counter := ModelDef.numstate + 1 to processnum - 1 do
         NumberofParams := NumberofParams + proc[counter].parameters;
  ParCount := NumberofParams;
end; // end of parcount function

{ This procedure supplies the derivatives of the state variables to the
  integrator. Since the integrator deals only with the values of the variables
  and not there names, units or the state field HoldConstant, this procedure
  copies the state values into a temporary state array and copies the value of
  HoldConstant into the temporary state array and passes this temporary state
  array to the procedure processes. }
PROCEDURE derivs(t:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
var
   i:integer;
   tempproc:processarray;
   tempstate:statearray;
begin
   tempstate := stat;  // Copy names, units and HoldConstant to tempstate
  // Copy current values of state variables into tempstate
   for i := 1 to ModelDef.numstate do tempstate[i].value := statevalue[i];
  // Calculate the process values
   processes(t,tdrive,tpar,tempstate,tempproc,false);
  // Put process values into dydt array to get passed back to the integrator.
   for i:= 1 to ModelDef.numstate do dydt[i]:=tempproc[i].value;
end;  // end of derivs procedure

end.
