{ This file creates and controls the main window of the modeling application.
  This window is called MainForm in the code. Describe the use of ReadytoRun and
  RunComplete here.}
unit frontend;

interface                      

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, stypes, ExtCtrls, TeeProcs, TeEngine, Chart, Mask, Printers;
 
    function ATrigger(period,ST,FN:double):double;
    
type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    MIFiles: TMenuItem;
    MISaveParam: TMenuItem;
    DlgOpenParam: TOpenDialog;
    DlgSaveParam: TSaveDialog;
    MIEdit: TMenuItem;
    MIEditParam: TMenuItem;
    MIEditStates: TMenuItem;
    MIChooseDrv: TMenuItem;
    MIOutputFile: TMenuItem;
    DlgOpenDriver: TOpenDialog;
    MIChooseParam: TMenuItem;
    MICalculate: TMenuItem;
    MIView: TMenuItem;
    MIOutTimeStep: TMenuItem;
    MIChart: TMenuItem;
    MIParamFile: TMenuItem;
    MIDriverFile: TMenuItem;
    MIEditDrv: TMenuItem;
    DlgSaveDriver: TSaveDialog;
    DlgSaveOutput: TSaveDialog;
    MISaveAsParam: TMenuItem;
    MIExit: TMenuItem;
    MIBar: TMenuItem;
    MITable: TMenuItem;
    DummyChart: TChart;
    ParamLabel: TLabel;
    DriverLabel: TLabel;
    OutputLabel: TLabel;
    StartLabel: TLabel;
    StopLabel: TLabel;
    StepLabel: TLabel;
    PresentLabel: TLabel;
    MICalcSS: TMenuItem;
    MIEditDrv2: TMenuItem;
    MEStartTime: TMaskEdit;
    MEStopTime: TMaskEdit;
    METimeStep: TMaskEdit;
    ParamBox: TEdit;
    DriverBox: TEdit;
    OutputBox: TEdit;
    BtnRun: TButton;
    PresentBox: TEdit;
    BtnReload: TButton;
    MICalTime: TMenuItem;
    MIViewOutFile: TMenuItem;
    DlgOpenOutput: TOpenDialog;
    DlgPrint: TPrintDialog;
    Run1: TMenuItem;
    MISpecialRun: TMenuItem;
    Help1: TMenuItem;
    MIAbout: TMenuItem;
    MIRuntoSS: TMenuItem;
    MINormalRun: TMenuItem;
    procedure ChooseParamFile(Sender: TObject);
    procedure ChooseDriver(Sender: TObject);
    procedure ChooseOutputFile(Sender: TObject);
    procedure BtnRunClick(Sender: TObject);
    procedure MIExitClick(Sender: TObject);
    procedure MISaveParamClick(Sender: TObject);
    procedure MISaveAsParamClick(Sender: TObject);
    procedure MIEditParamClick(Sender: TObject);
    procedure MIEditStatesClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure StartBoxChange(Sender: TObject);
    procedure StopBoxChange(Sender: TObject);
    procedure StepBoxChange(Sender: TObject);
    procedure BtnReloadClick(Sender: TObject);
    procedure MIOutTimeStepClick(Sender: TObject);
    procedure UpdateMainForm;
    procedure MIEditDrvClick(Sender: TObject);
    procedure MIAboutClick(Sender: TObject);
    procedure MICalcSSClick(Sender: TObject);
    procedure DisplayOutput(Sender: TObject);
    procedure MICalTimeClick(Sender: TObject);
    procedure MIViewOutFileClick(Sender: TObject);
    procedure CheckOutputStep(newvalue:double);
    procedure METimeStepKeyPress(Sender: TObject; var Key: Char);
    procedure MEStopTimeKeyPress(Sender: TObject; var Key: Char);
    procedure MEStartTimeKeyPress(Sender: TObject; var Key: Char);
    procedure MISpecialRunClick(Sender: TObject);
    procedure MINormalRunClick(Sender: TObject);
  private
    { Private declarations }
    initialstates:statearray;
    procedure ResetStatesNow(Sender: TObject);
  public
    { Public declarations }
   RunningModelBatch: Boolean;
   RunningSensitivity: Boolean;
   RunXRepeatedly:Boolean;
   RepeatDriveTime:Integer;
   ResetStates:Boolean;
   ResetStateTime:Integer;
 end;

var
  MainForm : TMainForm;
  time,last_time,next_time,time_start,Time_stop,time_step,CalTime : double;
  ModelDef : TModelDef;
  stat : statearray;
  drive,last_drive,next_drive : drivearray;
  par : paramarray;
  proc : processarray;
  paramfilename,driverfilename,outfilename : string;
  NeedToSavePar : Boolean = False;
  ReadyToRun : Boolean = False;
  RunComplete: Boolean = False;
  NewParamFile: Boolean;
  NewDriverFile: Boolean;
  outputstep:double;
  accuracy:double;
//  NumSpProc,NumSpPar:integer;     // MEL specific
//  numspp:integer;                 // MEL specific

implementation

uses fileio, Calculate, display, ReloadDlg, params, data,
     equations, integrator, note, aboutbox, trouble, ProgressBar, RunOptions;

{$R *.DFM}

function ATrigger(period,ST,FN:double):double;
var B,C,tr:double;
begin
 B:=1/(1-sin(pi*(0.5-(FN-ST)/period )));
 C:=1-B;
 Tr:=C+B*sin(pi*(2*time/period+0.5-(FN+ST)/period));
 if Tr<0 then Tr:=0;
 ATrigger:=Tr;
end;

{ This procedure sets up the modeling application. }
procedure TMainForm.FormCreate(Sender: TObject);
var
 tempstring: string;
begin
  ForceCurrentDirectory := True;
 // Set initial values for model timing
  accuracy := 1e-4;
  Time_start := strtofloat(MEStartTime.text);
  Time_stop := strtofloat(MEStopTime.text);
  Time_step := Strtofloat(METimeStep.text);
  outputstep := Time_step;
  CalTime := time_start;
// Set initial value for the number of species
//  numspp := 1;          // MEL specific

{ Call the counts procedure which defines the model structure. This procedure
  sets the names and units of the state variables, driver variables, process
  variables, and parameters. It also sets the values of the ModelDef structure
  which defines the number of state variables, driver variables, process
  variables and parameters.
}
  counts;
  next_drive := drive; // Initialize names and units of next_drive & last_drive
  last_drive := drive;
  MainForm.Caption := ModelDef.modelname;  // Set the MainForm caption
  tempstring := Application.Title;
  tempstring := lowercase(tempstring);
  if tempstring = 'modelbatch' then
     RunningModelBatch := True
  else
     RunningModelBatch := False;
  if tempstring = 'sensitivity' then
     RunningSensitivity := True
  else
     RunningSensitivity := False;
end;

{ This procedure enables the user to choose a parameter file using several
  different methods. The user can click Choose Parameter file from the menu,
  click on the label Parameter File, or type a filename directly in the box.
  If the user chooses using either the menu or the label, an open file dialog
  is displayed. }
procedure TMainForm.ChooseParamFile(Sender: TObject);
var
 oldparamfilename : string;
 i : integer;
begin
 oldparamfilename := paramfilename;
 // If the user typed directly in the box
 if Sender is TEdit then paramfilename := parambox.text  // Set paramfilename
 else if not RunningSensitivity then// The user either the menu or clicked on the label
   begin  // Show the open file dialog
           // First set the dialog box default filename to the current paramfile
      DlgOpenParam.filename := paramfilename;
          // If the user chooses OK in the dialog then set the new paramfilename
      if DlgOpenParam.execute then paramfilename := DlgOpenParam.filename;
   end;

 if (paramfilename <> oldparamfilename) or RunningSensitivity then
  begin
   if paramfilename <> '' then  // Be sure something was entered for the filename
    begin
      if not FileExists(paramfilename) then  // If the paramter file doesn't exist
        // Create a new empty parameter file.
       begin
        WriteParamFile(paramfilename,ModelDef.numparam,par,ModelDef.numstate,
                        stat);
        NewParamFile := True;
        for i := 1 to ModelDef.numstate do stat[i].value := 0;
        for i := 1 to ModelDef.numparam do par[i].value := 0;
       end
      else // Parameter does exist, so read it into the global arrays.
       begin
        if ReadParamFile(paramfilename,ModelDef.numparam,par,ModelDef.numstate,
                         stat) then
          NewParamFile := False
        else
         begin
          for i := 1 to ModelDef.numstate do stat[i].value := 0;
          for i := 1 to ModelDef.numparam do par[i].value := 0;
          paramfilename := '';
         end;
       end;
    end;
   RunComplete := False;
   Readytorun := false;   // Can't run the model.
  end;

 UpdateMainForm;  // Update the form
end;

{ This procedure enables the user to choose a Driver file using several
  different methods. The user can click Choose Driver file from the menu,
  click on the label Driver File, or type a filename directly in the box.
  If the user chooses using either the menu or the label, an open file dialog
  is displayed. }
procedure TMainForm.ChooseDriver(Sender: TObject);
var
 olddriverfilename : string;
 i:integer;
begin
 olddriverfilename := driverfilename;
// If the user typed directly in the edit box
 if Sender is TEdit then driverfilename:=DriverBox.text  // Set the driverfile
 else  // User used the menu or clicked on the Driver label
  begin   // Show the open file dialog
    // First set the default filename to the current driverfile
   DlgOpenDriver.filename := driverfilename;
    // Show the dialog and if the user clicks OK, set the new driver filename
   if DlgOpenDriver.execute then
    begin
//      if FileExists(DlgOpenDriver
      driverfilename := DlgOpenDriver.filename;
    end;
  end;

 if driverfilename <> olddriverfilename then
  begin
   if driverfilename <> '' then
     if not FileExists(driverfilename) then
      begin
       NewDriverFile := True;
       for i := 1 to ModelDef.numdrive do drive[i].value := 0;
       OpenDriverFile(driverfilename,ModelDef.numdrive,drive,flwrite);
       WriteDriverFile(time,ModelDef.numdrive,drive);
       CloseDriverFile;
      end
     else
       NewDriverFile := False;
   ReadytoRun := False; // This forces the edit boxes on the mainform to be update with the new filename
   RunComplete := False;
  end;

 UpdateMainForm;
end;

{ This procedure enables the user to choose an Output file using several
  different methods. The user can click Choose Output file from the menu,
  click on the label Output File, or type a filename directly in the box.
  If the user chooses using either the menu or the label, an open file dialog
  is displayed. }
procedure TMainForm.ChooseOutputFile(Sender: TObject);
var
 oldoutfilename : string;
begin     // If the user typed directly in the edit box
 oldoutfilename := outfilename;

 if Sender is TEdit then  Outfilename := OutputBox.text  // Set the output file
 else   // User used the menu or clicked on the Output label
  begin  // Show the Save file dialog
    // First set the default filename to the current output file.
   DlgSaveOutput.filename := outfilename;
   // Show the dialog and if the user chooses OK, set the outfilename to the new file.
   if DlgSaveOutput.execute then outfilename := DlgSaveOutput.filename;
  end;

{  if outfilename <> oldoutfilename then
  begin
  if outfilename <> '' then  // Make sure a value was entered for the filename
    begin
     if FileExists(outfilename) then  // If the file already exists
      // Confirm that the user wants to overwrite the existing file.
       if MessageDlg('Overwrite existing output file, ' + outfilename + '?',
                  mtConfirmation, [mbYes,mbNo], 0) = mrNo then
        begin
          outfilename := '';  // Set the outfile to no value
        end;
    end;              
  end;                 }
 RunComplete := False;
 ReadytoRun := False;  // Can't run without an output file.
 UpdateMainForm; // Update the form.
end;

{ This procedure saves the current values of the state variables and the
  parameters to the current parameter file. }
procedure TMainForm.MISaveParamClick(Sender: TObject);
begin
 WriteParamFile(paramfilename,ModelDef.numparam,par,ModelDef.numstate,stat);
 RunComplete := False;
 NewParamFile := False;
 UpdateMainForm;
 Beep;
end;

{ Procedure to save the state variables and parameters to a new parameter file. }
procedure TMainForm.MISaveAsParamClick(Sender: TObject);
begin  // Show the Save dialog
with DlgSaveParam do
 begin
  // Set the default filename in save dialog to the current parameter file.
  filename := paramfilename;
  if execute then   // If the user chooses OK in the save dialog.
   begin
    paramfilename:=filename; // Set the paramfilename to the new filename.
    // Save the state variables and parameters to the current file.
    WriteParamFile(paramfilename,ModelDef.numparam,par,ModelDef.numstate,stat);
    // Set Ready to Run to false so the the edit boxes on the form will be
    // updated when UpdateMainForm is called.
    NewParamFile := False;
    ReadytoRun := False;
    RunComplete := False;
    UpdateMainForm;  // Update the form.
   end;
 end;
end;

{ Procedure to allow the user to edit the parameter values. This procedure calls
  the Parameter Form defined in the params.pas unit. While the user is editing
  the parameters the MainForm is disabled. }
procedure TMainForm.MIEditParamClick(Sender: TObject);
begin
{ Show the parameter form. If the user clicked OK on the parameter form,
  meaning they changed the parameter values, update this form. }
 if FmParameter.showmodal = mrOK then      //fix
  begin
// Parameters have been changed so any previous model runs are invalid.
   RunComplete := False;
   UpdateMainForm;   // Update the form.
  end;
end;

{ Procedure to allow the user to edit the state variables. This procedure calls
  the Data Form defined in the data.pas unit. While the user is editing
  the state variables the MainForm is disabled. }
procedure TMainForm.MIEditStatesClick(Sender: TObject);
begin
 DataForm.ShowStates;  // Show the data form containing state variables
 UpdateMainForm;  // Update the form.
end;

{ Procedure to allow the user to edit the driver variables. This procedure calls
  the Note Form defined in the note.pas unit. While the user is editing
  the drivers the MainForm is disabled. }
procedure TMainForm.MIEditDrvClick(Sender: TObject);
begin
{ Show the note form. If the user clicked OK on the note form,
  meaning they changed the driver values, update this form. }
 if FmNote.ShowModal = mrOK then
  begin
// Drivers have been changed so any previous model runs are invalid.
   RunComplete := False;
// User may have changed driver files, so force the edit boxes and menu to be updated.
   ReadytoRun := False;
   UpdateMainForm;  // Update the form.
  end;
end;

{ Procedure to process changes made to the start time mask edit box. The mask
  edit box only allows numeric input with up to 5 digits. Values entered must be
  integers. To change the allowed values modify the mask using the object
  inspector. }
procedure TMainForm.StartBoxChange(Sender: TObject);
begin
 if (MEStartTime.text <> '') and (MeStartTime.Modified) then 
  begin
   Time_Start := strtofloat(MEStartTime.text); // Set the start time of the model.
   ReadytoRun := False;  // Force checking of start and stop times in UpdateMainForm.
   RunComplete := False;  // Previous runs are invalid.
   UpdateMainForm;   // Update the form.
  end;
end;

{ Procedure to process changes made to the start time mask edit box. The mask
  edit box only allows numeric input with up to 5 digits. Values entered must be
  integers. To change the allowed values modify the mask using the object
  inspector. }
procedure TMainForm.StopBoxChange(Sender: TObject);
begin
 if (MEStopTime.text <> '') and (MeStopTime.Modified) then
  begin
   Time_Stop := strtofloat(MEStopTime.text); // Set the stop time of the model.
   ReadytoRun := False; // Force checking of start and stop times in UpdateMainForm.
   RunComplete := False;  // Previous runs are invalid.
   UpdateMainForm;   // Update the form.
  end;
end;

{ Procedure to process changes made to the step time mask edit box. The mask
  edit box only allows numeric input with up to 5 digits. Values entered must be
  integers. To change the allowed values modify the mask using the object
  inspector. }
procedure TMainForm.StepBoxChange(Sender: TObject);
var
 newstep:double;
begin
 if (METimeStep.text <> '') and (MeTimeStep.Modified) then
  begin
   newstep := strtofloat(METimeStep.text);
{   if newstep >= Time_Stop then
    begin
      messagedlg('Time Step must be less than Stop Time. Please reenter.',
          mtWarning,[mbOK],0);
      MeTimeStep.text := floattostr(Time_Step);
      MeTimeStep.SelectAll;
    end
   else          }
      Time_Step := newstep; // Set the time step of the model.

   ReadytoRun := False;  // Fix - do I need this?
   RunComplete := False; // Previous runs are invalid.
   UpdateMainForm;  // Update the form.
  end;

  // Check that the output step is greater than the current time step, if not,
  // increase it.
  try
    newstep := outputstep;
    CheckOutputStep(newstep);
  except
    on E:EOutputstepTooSmall do
     begin
      outputstep := time_step;
      MessageDlg(E.Message + ' Output time step increased to time step.',
         mtInformation, [mbOK], 0);
     end;
    on E:EOutputstepTooLarge do
     begin
      outputstep := time_stop - time_start;
      MessageDlg(E.Message + ' Output time step decreased to run length.',
         mtInformation, [mbOK], 0);
     end;
    on E:EOutputstepNotMultiple do
     begin
      outputstep := time_step;
      MessageDlg(E.Message + ' Output time step changed to current time step.',
         mtInformation, [mbOK], 0);
     end;
  end;

  FmDisplayData.DisplayStep := outputstep;
end;

{ This procedure allows the user to change the display time step from the
  default value. The display time step is the time step with which data is
  stored in the stringgrid on the display form. It does not effect the time
  step of the output in the output file. It is for display purposes only.}
procedure TMainForm.MIOutTimeStepClick(Sender: TObject);
begin
  repeat  // Ask the user for a output time step, until the value entered is
          // greater than zero.
    outputstep := strtofloat(InputBox('Output Time Step',
       'Enter the time step for the output data.',
       floattostr(outputstep)));
  until outputstep > 0 ;
            // what if user clicks cancel - change this
  try
    CheckOutputStep(outputstep);
  except
    on E:EOutputstepTooSmall do
     begin
      outputstep := time_step;
      MessageDlg(E.Message + ' Output time step increased to time step.',
         mtInformation, [mbOK], 0);
     end;
    on E:EOutputstepTooLarge do
     begin
      outputstep := time_stop - time_start;
      MessageDlg(E.Message + ' Output time step decreased to run length.',
         mtInformation, [mbOK], 0);
     end;
    on E:EOutputstepNotMultiple do
     begin
      outputstep := time_step;
      MessageDlg(E.Message + ' Output time step changed to current time step.',
         mtInformation, [mbOK], 0);
     end;
  end;

  FmDisplayData.DisplayStep := outputstep;
  RunComplete := False; // Previous runs are invalid.
  UpdateMainForm;  // Update the form.
end;

{ This procedure shows the output from the model run in table or chart format.
  It uses the form defined in display.pas. }
procedure TMainForm.DisplayOutput(Sender: TObject);
begin
 if (Sender as TMenuItem).Name = 'MITable' then
  FmDisplayData.DisplayStyle := dsTable // Set the type of display.
 else
  FmDisplayData.DisplayStyle := dsChart;
 FmDisplayData.showmodal; // Show the Display Form.
end;

{ Show an about box containing general information about the model. }
procedure TMainForm.MIAboutClick(Sender: TObject);
begin
 FmAbout.ShowModal;
end;

{ This procedure presents a dialog which allows the user to reread the state
  variables and/or parameters from the parameter form. It uses the Reload
  dialog defined in the ReloadDlg.pas unit.}
procedure TMainForm.BtnReloadClick(Sender: TObject);
begin  // Show the Reload Dialog
if paramfilename <> '' then
 begin
  if DlgReload.ShowModal = mrOK then  // If the user clicked OK, values were reloaded
   begin
    RunComplete := False; // so previous runs are invalid.
    UpdateMainForm;  // Update the form.
   end;
 end
else
  MessageDlg('Invalid parameter file. No values reloaded.',mtWarning,[mbOK],0);
end;

// This procedure is used to update the controls on the Main Form, i.e. enabling
// and disabling them when appropriate.
procedure TMainForm.UpdateMainForm;
begin
 if not ReadytoRun then
  begin
//   time := 0;
//   PresentBox.Text := floattostr(time);
   ParamBox.text := paramfilename; // save it's name in the paramfilename variable
   if paramfilename <> '' then // if a parameter file has been chosen
    begin
     MISaveParam.Enabled := true;  // enable the save parameter file menu item
    end;
     // if a driver variable has been chosen, save it's name to driverfilename
   if driverfilename <> '' then DriverBox.text := driverfilename;
    // if an output file name has been chosen, save it's name to outfilename
   if outfilename <> '' then OutputBox.text := outfilename;
// If all files have been chosen then enable the run button and calculate menu.
   if (DriverBox.text <> '') and (ParamBox.text <> '') then
     begin
//       MICalculate.enabled := true; // Enable the calculate menu
       if (OutputBox.text <> '') and (Time_stop > Time_start) then
         begin
          ReadytoRun := true;
          BtnRun.enabled := true;  // Enable the run button
         end;
     end;
  end;
 if RunComplete then  // If a run is complete
  begin
   MIChart.Enabled := true;  // Enable the chart output menu item
   MITable.Enabled := true;  // Enable the table output menu item
   last_time := 0;
   next_time := 0;
  end
 else
  begin // Run is not complete or previous run is now invalid
   MIChart.Enabled := False;  // Disable the chart output menu item.
   MITable.Enabled := False;  // Disable the table output menu item.
   PresentBox.Text := floattostr(0); // Set the present time to 0
//   if Application.ExeName = 'modelbatch.exe' then
//     FmDisplayData.NeedtoReadData := True;// Tell display form the new data will need to be read in.
  end;
 if MainForm.ActiveControl is TEdit then
   if MainForm.ActiveControl.Name = 'ParamBox' then
    MainForm.ActiveControl := DummyChart;
end;

// This procedure controls the actual running of a model scenario.
procedure TMainForm.BtnRunClick(Sender: TObject);
var
  j, counter, outputfreq:integer;
  nok,nbad:integer;
  tempstat:yValueArray;
  drivetime, dailystep, remainder:double;
begin
  if (not RunningModelBatch) and (not RunningSensitivity) then
   begin
    FmProgress.Show;
    MainForm.Enabled := False;
    FmDisplayData.NeedtoReadData := true;
   end;
  initialstates := stat;
  last_time := 0;
  next_time := 0;
  OpenDriverFile(driverfilename,ModelDef.numdrive,drive,flread);
  OpenOutputFile(Outfilename,ModelDef.numstate,stat,ModelDef.numprocess,
                                                                proc,flWrite);
  time := time_start;
  drivetime := time;
// Variables used to keep track of when to write to output file
  counter := 1;
  outputfreq := round(outputstep/time_step);
// Calculate and write out the initial state of system.
  GetCurrentDrivers(time,drive);
  processes(time,drive,par,stat,proc,false);    
  dailystep := time_step;         // not MEL
//  WriteOutputFile(Time,ModelDef.numstate,stat,ModelDef.numprocess,proc);
//  randomize;      // MEL specific
 try try
  while time + time_step <= time_stop do
    begin
      for j:=1 to ModelDef.numstate do tempstat[j] := stat[j].value;
      odeIntegrator(tempstat, modeldef.numstate, drivetime, drivetime+time_step,
           accuracy, dailystep, 0.000001*time_step, nok, nbad, drive, par);
      for j:=1 to ModelDef.numstate do stat[j].value := tempstat[j];
      processes(drivetime+time_step,drive,par,stat,proc,true);
      if counter = outputfreq then
       begin
         WriteOutputFile(time+time_step,ModelDef.numstate,stat,ModelDef.numprocess,proc);
         counter := 0;
       end;
      if (not RunningModelBatch) and (not RunningSensitivity) then
           FmProgress.RunProgressBar.Position :=
                        (round(time + time_step - time_start)*100)
                           div round(time_stop - time_start);
      if (time<time_stop) and (time+time_step>time_stop)
               then time_step:=time_stop-time; // Necessary? Gives a funny end maybe
                                               // should time really be time+time_step?
// Reset State variables option
      if ResetStates then
        begin
          remainder := time/ResetStateTime - int(time/ResetStateTime);
          if remainder < time_step/ResetStateTime then MainForm.ResetStatesNow(BtnRun);
        end;
      time := time + time_step;
      drivetime :=  drivetime + time_step;
// Repeat drivers option
      if RunXRepeatedly and (drivetime > RepeatDriveTime) then
       begin
        last_time := 0;     next_time := 0;
        drivetime := time_start;
       end;
      GetCurrentDrivers(drivetime,drive);
      counter := counter + 1;
    end;
 finally
  CloseDriverFile;
  CloseOutputFile;
  time_step := strtofloat(METimeStep.edittext);
  RunComplete := True;
  if (not RunningModelBatch) and (not RunningSensitivity) then
   begin
    FmDisplayData.FileName := outfilename;
    UpdateMainForm;
    MainForm.Enabled := True;
    PresentBox.Text := floattostrf(Time,ffgeneral,5,1);
    FmProgress.Hide;
   end;
 end;
 except
  on EUserCancel do;
  on E: Exception do
   begin
   if (not RunningModelBatch) and (not RunningSensitivity) then
    begin
     FmTrouble.MmError.Lines.Clear;
     FmTrouble.MmError.Lines.Add(E.Message);
     FmTrouble.ShowModal;
    end
    else
     raise;
   end;
 end;
end;

procedure TMainForm.MIExitClick(Sender: TObject);
var
  answer:word;
begin
  if NeedToSavePar = True then
        begin
          answer := MessageDlg('Save parameter file before exiting?',
                 mtConfirmation, mbYesNoCancel, 0);
          if answer <> mrCancel then
           if answer = mrYes then
             begin
              DlgSaveParam.Filename := paramfilename;
              if DlgSaveParam.execute then
                WriteParamFile(paramfilename,ModelDef.numparam,par,
                                  ModelDef.numstate,stat);
             end
           else
             close;
        end
      else close;
end;

procedure TMainForm.MICalTimeClick(Sender: TObject);
begin
// Ask the user for the time to calculate a steady state at.
  CalTime := strtofloat(InputBox('Steady State Time',
    'Enter the time from the driver file at which to calculate a steady state.',
    floattostr(CalTime)));
  RunComplete := False;
  FmDisplayData.NeedtoReadData := True; // Tell the display form to reread in the output data.
end;

procedure TMainForm.MICalcSSClick(Sender: TObject);
begin
 FmCalculate.showmodal;
end;

procedure TMainForm.MIViewOutFileClick(Sender: TObject);
begin
 if DlgOpenOutput.execute then
  begin
   if DlgOpenOutput.filename <> '' then
     FmDisplayData.Filename := DlgOpenOutput.filename;
   FmDisplayData.NeedtoReadData := True;
   RunComplete := True;
   UpdateMainForm;
  end;
end;

procedure TMainForm.CheckOutputStep(newvalue:double);
begin
 if newvalue < time_step then
  begin
   raise EOutputStepTooSmall.Create('Output time step is less than simulation '
          + 'time step.');
  end;

 if newvalue > time_stop - time_start then
  begin
   raise EOutputStepTooLarge.Create('Output time step is greater than run length.');
  end;

 if abs(newvalue/Time_step - round(newvalue/Time_step)) > 0.001 then
  begin
    raise EOutputStepNotMultiple.Create('Output time step must be a multiple '
           + 'of simulation time step.');
  end;
end;

procedure TMainForm.METimeStepKeyPress(Sender: TObject; var Key: Char);
begin
 if {(Sender is TMaskEdit) and }(Key = Chr(13)) then StepBoxChange(Sender);
end;

procedure TMainForm.MEStopTimeKeyPress(Sender: TObject; var Key: Char);
begin
 if (Sender is TMaskEdit) and (Key = Chr(13)) then StopBoxChange(Sender);
end;

procedure TMainForm.MEStartTimeKeyPress(Sender: TObject; var Key: Char);
begin
 if (Sender is TMaskEdit) and (Key = Chr(13)) then StartBoxChange(Sender);
end;

procedure TMainForm.MISpecialRunClick(Sender: TObject);
begin
 FmRunOptions.ShowModal;
end;

procedure TMainForm.MINormalRunClick(Sender: TObject);
begin
 RunXRepeatedly := False;
 ResetStates := False;
 MINormalRun.Checked := True;
 RunComplete := False;
 Readytorun := False;
end;

procedure TMainForm.ResetStatesNow(Sender: TObject);
var
 i: integer;
begin
 for i := 1 to ModelDef.numstate do
   begin
     if stat[i].reset then stat[i].value := initialstates[i].value;
   end;
end;

end.
