{ This file defines new variable types and sets the limits on the various arrays
  used in the model code. All units should include this file. }
unit stypes;

interface

uses sysutils;

const
  maxstate = 35;   // The maximum number of state variables. The state variable
                   // array contains maxstate elements.
  maxdrive = 10;   // The maximum number of driver variables.
  maxprocess = 200;  // The maximum number of process variables.
  maxparam = 100;     // The maximum number of parameters.
  maxresults = 20;
  stringlength = 25;

type
  processtype = (ptGroup1, ptGroup2, ptGroup3, ptGroup4, ptGroup5);
//  processcolor = (clGreen, clTeal, clAqua, clBlue, clGray);
  TModelDef = record   // A structure used to hold the basic information
   modelname:string;   // about the model. Modelname is the name of
                       // model and appears on the title bar in the MainForm.
   versionnumber:string; // Used to keep track of the version number of the model.
   timeunit:string;    // Timeunit is the time step of the model, ie day, year.
   numdrive:integer;   // Numparam, numdrive, numstate and numprocess are the
   numstate:integer;  // total number of parameters, drivers, state variables,
   numprocess:integer;   // and processes, respectively.
   numparam:integer;
   end;

  Tparamvariable = record  // A structure to hold a parameter; it's name, value,
     name:string;         // and units. The name and units are for information
     value:double;        // only. They do not affect the running of the model.
     units:string;        // This structure type is also used for the
     symbol:string;       // driver variables.
  end;                     

  Tstatevariable = record  // A structure to hold a state variable; the name,
     name:string;          // value and units. As with Tparamvariable the name
     value:double;         // and units are for information only. HoldConstant
     units:string;         // determines if the state variable is held constant
     symbol:string;        // during the model run, i.e. dstatedt = 0.
     HoldConstant:Boolean;
     Reset:Boolean;
  end;

  Tprocessvariable = record  // A structure to hold a process variable; the
     name:string;            // name, value, and units. As above, the name and
     value:double;           // units are for information only. Parameters is
     units:string;           // number of parameters associated with this
     symbol:string;          // particular process. Every parameter must be
     parameters:integer;     // associated with a process. Ptype determines the
     ptype:processtype;      // color of the tab in the edit parameters page.
                             // Isdiscrete identifies
     Isdiscrete:Boolean      // whether or not this process should be integrated
  end;                       // or not. At this time Isdiscrete is used for
                             // information only.

  Tallstates = record
     Cl:Tstatevariable;
     Cw:Tstatevariable;
     Cr:Tstatevariable;
     Nl:Tstatevariable;
     Nw:Tstatevariable;
     Nr:Tstatevariable;
  end;

  TTreatResults = record
     Treatment: string;
     Year: integer;
     States: TAllStates;
  end;


// State variable array types. yValueArray, glindx and glpbynp are used during
// integration. They hold temporary values of the state variables and therefore
// don't need to be of type Tstatevariable. This saves memory. However they
// must be the same size as statearray.
  yValueArray=array[1..maxstate] of double;
  glindx = ARRAY [1..maxstate] OF integer;
  glnpbynp = ARRAY [1..maxstate,1..maxstate] OF double;
  statearray=array[1..maxstate] of Tstatevariable;
// Driver variable array type. Note that the type is Tparamvariable.
  drivearray=array[1..maxdrive] of Tparamvariable;
// Parameter array type.
  paramarray=array[1..maxparam] of Tparamvariable;
// Process variable array type.
  processarray=array[1..maxprocess] of Tprocessvariable;
  resultarray = array[1..maxresults] of TTreatResults;

// The types of data which can be displayed in the dataform
  TDataType = (dtState,dtProcess);
// The possible axis' in the chart on the display form
  TAxis = (axLeft, axBottom);
// The display options for the output data in the display form
  TDisplayStyle = (dsChart, dsTable);
// The axis type for the chart on the display form
  TAxisType = (tyLinear, tyLog);
// The type of variable being looked for in the arrays
  TVarType = (vtDriver, vtState, vtParameter, vtProcess);

  ERunTimeError = class(Exception);
  EIntegratorError = class(ERunTimeError);
  ETooManySteps = class(EIntegratorError);
  EStepTooSmall = class(EIntegratorError);
  EUserCancel = class(ERunTimeError);
  ECalculateError = class(EMathError);
  ECalcInitialProc = class(ECalculateError);
  ECalcNewValue = class(ECalculateError);
  ECalcNewProcess = class(ECalculateError);
  EStepError = class(Exception);
  EOutputStepTooSmall = class(EStepError);
  EOutputStepTooLarge = class(EStepError);
  EOutputStepNotMultiple = class(EStepError);
implementation

end.
 